/**
 * These classes have been developed for the DFSLog system.
 *
 * @author N. Kornweibel nkornwei@eso.org
 *
 * TO DO:
 * need @revision, and SNIFF info here.
 * 
 */
package org.eso.dfslog;

import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;

// JDBC
import java.sql.*;

/**
 * LogViewer:
 *
 * A simple frame for displaying a ScrollableLogTable with a specified
 * initial select statement. The statement is editable, as well there is
 * control over the frequency of automatic refreshing of the view.
 * 
 * Menu controls allow for report generation and exiting the window.
 */

public class LogViewer extends JFrame implements ActionListener {
    private ScrollableLogTable scrollLogTable;
    private ConnectionServer conn;
    private String currentStatement;
    private JTextField selectEditor;

    private ReportViewer rv = null;

    private JComboBox pollPeriodCombo;
    private SelectTableModel model;

    private javax.swing.Timer pollTimer = null;

    private static final String quitLabel = "Quit";
    private static final String fileLabel = "File";
    private static final String reportLabel = "Report";
    private static final String entireViewLabel = "Entire View";
    private static final String selectedRowsLabel = "Selected Rows";
    private static final String refreshLabel = "Refresh";

    private String lastUpdate = null;

    private MessageDisplay messageDisplay = null;

    private JButton refreshButton = null;
    private JLabel lastUpdateLabel = null;

    private RefreshRate[] comboOptions;

    public LogViewer(String title, ConnectionServer c, String sqlSelect) {
        super(title);
        conn = c;

        scrollLogTable = new ScrollableLogTable();
        messageDisplay = new MessageDisplay(scrollLogTable.getTable());


        comboOptions = new RefreshRate[8];
        comboOptions[0] =
            new RefreshRate("No periodic poll for new messages", 0);
        comboOptions[1] =
            new RefreshRate("Poll for new messages every 10 seconds", 10);
        comboOptions[2] =
            new RefreshRate("Poll for new messages every 20 seconds", 20);
        comboOptions[3] =
            new RefreshRate("Poll for new messages every 30 seconds", 30);
        comboOptions[4] =
            new RefreshRate("Poll for new messages every 1 minute", 60);
        comboOptions[5] =
            new RefreshRate("Poll for new messages every 2 minutes", 120);
        comboOptions[6] =
            new RefreshRate("Poll for new messages every 5 minutes", 300);
        comboOptions[7] =
            new RefreshRate("Poll for new messages every 10 minutes", 600);

        pollPeriodCombo = new JComboBox(comboOptions);
        pollPeriodCombo.addActionListener(this);
        pollPeriodCombo.setSelectedIndex(0);
        selectEditor = new JTextField(currentStatement);
        selectEditor.addActionListener(this);

        // add the messageDisplay as a ListSelectionListener.
        ListSelectionModel lsm = scrollLogTable.getTable().getSelectionModel();
        if (lsm != null) {
            lsm.addListSelectionListener(messageDisplay);
        }
        

        MenuBar mb = new MenuBar();
        addMenuOptions(mb);
        setMenuBar(mb);
        setPollRate(0);

        Box vb = Box.createVerticalBox();
        vb.add(Box.createVerticalStrut(20));
        vb.add(selectEditor);
        JPanel p = new JPanel(new BorderLayout(), true);
        vb.add(p);
        p.add("Center", pollPeriodCombo);
        JPanel p2 = new JPanel();
        lastUpdateLabel = new JLabel(getLastUpdateTimeString());
        refreshButton = new JButton(refreshLabel);
        refreshButton.addActionListener(this);
        p2.add(refreshButton);
        p2.add(lastUpdateLabel);
        p.add("East", p2);

        // need another JPanel - reuse p2
        p2 = new JPanel(new BorderLayout(), true);
        p2.add("Center", scrollLogTable);
        p2.add("South", messageDisplay);
        // getContentPane().add(scrollLogTable, "Center");
        getContentPane().add(p2, "Center");
        getContentPane().add(vb, "North");

        setupTable(sqlSelect);
    }




    private void addMenuOptions(MenuBar mb) {
        Menu m = new Menu(fileLabel);
        MenuItem mi = new MenuItem(refreshLabel);
        mi.addActionListener(this);
        m.add(mi);
        mi = new MenuItem(quitLabel);
        mi.addActionListener(this);
        m.add(mi);
        mb.add(m);
        m = new Menu(reportLabel);
        mi = new MenuItem(selectedRowsLabel);
        mi.addActionListener(this);
        m.add(mi);
        mi = new MenuItem(entireViewLabel);
        mi.addActionListener(this);
        m.add(mi);
        mb.add(m);
    }




    private String getLastUpdateTimeString() {
        if (lastUpdate == null) {
            recordUpdateTime();
        }
        return "Last Update: "+lastUpdate;
    }




    private void recordUpdateTime() {
        SimpleDateFormat formatter
            = new SimpleDateFormat("dd MMM yyyy HH:mm:ss");
        lastUpdate = formatter.format(new java.util.Date());
    }



    private void displayUpdateTime() {
        recordUpdateTime();
        lastUpdateLabel.setText(getLastUpdateTimeString());
    }




    private void setupTable(String sqlStmt) {
        if (sqlStmt == null) {
            userMessage("Null SQL statement provided");
            return;
        }
        model = new SelectTableModel(conn, "dfslog", sqlStmt);
        if (model.isValid()) {
            scrollLogTable.setModel(model);
            // clear the message display
            messageDisplay.valueChanged(null);
            currentStatement = sqlStmt;
            selectEditor.setText(currentStatement);
        } else {
            userMessage("Failed to read data: "+model.getLastError());
        }
    }




    // implementation of the ActionListener interface
    public void actionPerformed(ActionEvent a) {
        if (a.getSource() == pollTimer) {
            // stop the timer while we process the event.
            pollTimer.stop();
            if (model.update() == true) {
                displayUpdateTime();
            }
            // restart the timer.
            pollTimer.restart();
        } else if (a.getSource() == pollPeriodCombo) {
            RefreshRate r = (RefreshRate)pollPeriodCombo.getSelectedItem();
            if (r != null) {
                setPollRate(r.getRefreshRate());
            }
        } else if (a.getSource() == selectEditor) {
            String comString = selectEditor.getText();
            comString.trim(); 
            String c = comString;
            c.toLowerCase();
            if (c.startsWith("select") == false) {
                userMessage("Statement must be a select");
                selectEditor.setText(currentStatement);
            } else {
                setupTable(comString);
            }
        } else if (a.getActionCommand() == refreshLabel) {
            // menu or button pressed.
            if (model.update() == true) {
                displayUpdateTime();
            }
        } else if (a.getActionCommand() == quitLabel) {
            if (pollTimer != null) {
                pollTimer.stop();
            }
            setVisible(false);
            dispose();
        } else if (a.getActionCommand() == entireViewLabel) {
            generateReport(true);
        } else if (a.getActionCommand() == selectedRowsLabel) {
            generateReport(false);
        }
    }




    private void userMessage(String msg) {
        JOptionPane.showMessageDialog(this, msg);
    }




    private void setPollRate(int seconds) {
        if (pollTimer == null) {
            pollTimer = new javax.swing.Timer(0, this);
            pollTimer.stop();
        }
        if (seconds == 0) {
            pollTimer.stop();
        } else {
            pollTimer.setDelay(seconds*1000);
            pollTimer.setInitialDelay(seconds*1000);
            pollTimer.restart();
        }
    }




    private void generateReport(boolean entireView) {
        Report rep = null;
        int proceed = JOptionPane.YES_OPTION;
        if (entireView) {
            int numRows = 0;
            JTable t = scrollLogTable.getTable();
            if (t != null ) {
                numRows = t.getRowCount();
            }
            if (numRows == 0) {
                proceed = JOptionPane.showConfirmDialog(this,
                    "This report will contain 0 messages\n"+
                    "are you sure you want to continue?",
                    "Please Confirm",
                    JOptionPane.YES_NO_OPTION);
            } else if (numRows > 500) {
                // large reports are time consuming and difficult to
                // handle - prompt the user for confirmation of action.
                proceed = JOptionPane.showConfirmDialog(this,
                    "This report will be large (over 500 messages)\n"+
                    "are you sure you want to continue?",
                    "Please Confirm",
                    JOptionPane.YES_NO_OPTION);
            }
            if (proceed == JOptionPane.YES_OPTION) {
                rep = new ConnServerSQLReport(conn, currentStatement);
            }
        } else {
            JTable t = scrollLogTable.getTable();
            int rows[] = t.getSelectedRows();
            if (rows.length == 0) {
                proceed = JOptionPane.showConfirmDialog(this,
                    "This report will contain 0 messages\n"+
                    "are you sure you want to continue?",
                    "Please Confirm",
                    JOptionPane.YES_NO_OPTION);
            } else if (rows.length > 500) {
                // large reports are time consuming and difficult to
                // handle - prompt the user for confirmation of action.
                proceed = JOptionPane.showConfirmDialog(this,
                    "This report will be large (over 500 messages)\n"+
                    "are you sure you want to continue?",
                    "Please Confirm",
                    JOptionPane.YES_NO_OPTION);
            }
            if (proceed == JOptionPane.YES_OPTION) {
                int rowCount = rows.length;
                int minRow = 0;
                if (rowCount > 0) {
                    minRow = rows[0];
                }
                int i = 1;
                while (i < rowCount) {
                    if (minRow > rows[i]) minRow = rows[i];
                    i++;
                }
                rep = new TableModelReport(t.getModel(), minRow, rowCount);
            }
        }
 
        if (proceed != JOptionPane.YES_OPTION) {
            // the user has cancelled the report.
            return;
        }
 
        // generate and display the report.
        // display the wait cursor while the new window is built.
        // Cursor currentCursor =  getCursor();
        // setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

        ReportViewer r;
        if (entireView) {
            r = new ReportViewer("Report of entire view", rep);
        } else {
            r = new ReportViewer("Report of selected rows", rep);
        }
        r.pack();
        r.setSize(1100, 500);
 
        // move it away from the top LH corner.
        r.setLocation(160, 200);
        r.setVisible(true);

        // display the original cursor
        // setCursor(currentCursor);
    }





    private class RefreshRate {
        private int rateInSeconds;
        private String label;

        public String toString() {
            return label;
        }

        RefreshRate(String l, int r) {
            label = l;
            rateInSeconds = r;
        }

        int getRefreshRate() {
            return rateInSeconds;
        }

        void setRefreshRate(int r) {
            rateInSeconds = r;
        }
    }

} // end of class LogViewer.

// end of file LogViewer.java

