/**
 * These classes have been developed for the DFSLog system.
 *
 * @author N. Kornweibel nkornwei@eso.org
 *
 * TO DO:
 * need @revision, and SNIFF info here.
 * 
 */
package org.eso.dfslog;

import javax.swing.table.*;
import java.io.*;
import java.util.*;
import java.text.*;

// JDBC imports
import java.sql.*;

/**
 * SelectTableModel:
 *
 * A TableModel which is built from the results of a SELECT JDBC
 * statement. In its constructor is required a Statement and execution
 * string for receiving the data.
 *
 * The statement is closed after each read - if this proves to be
 * inefficient, it will be improved. It is done in this manner for
 * simplicity and to avoid the possibility that the statement (after a long
 * period of dis-use) is closed on the server side.
 *
 * As the statement is re-executed to get new data, the table may well
 * have changed since the previous read and hence the class must handle
 * any form of results from the execution - more or less rows and
 * different data.
 *
 * At this point, structural changes in the table are not handled.
 * 
 * All SQL (and most other) exceptions are handled within this class.
 *
 * The update() method informs the model to flush its data cache and
 * re-fill it from the database, and is used to update the table with
 * new data from the database.
 */


class SelectTableModel extends AbstractTableModel {

    // Control variables of the Model

      /**
       * size of the cached rows.
       */
    private int cacheSize = 500;

      /**
       * Determines how close a getValue() call approaches either end 
       * of the cache before the cache "slides" to read new data 
       * - it is a # of rows.
       */
    private int dataReadWarning = 50;

      /**
       * data read size - how much data we read in from the database
       * when the cache slides.
       */
    private int cacheMoveSize = 250;

      /**
       * how much over estimation of the database size do we make.
       */
    private int rowNumberOverEstimate = 150;

      /**
       * Controls whether to read in data when necessary, or only 
       * when requested in the update() call.
       */
    private boolean manualUpdate = false;

    private int maxNumberCols;

    private ConnectionServer dbServer = null;

    
    // use for the column definition
    private ColumnEntry columns[] = null;
    private String databaseName = null;
    private String selectStatement = null;

    // is the model valid - was it initialized correctly.
    private boolean modelValid = false;
    private String lastError;

    /**
     * Sliding data cache
     */
    private SlidingArray rowsStored;

    /**
     * number of rows currently read.
     */
    private int knownMaxRow;

    private boolean dataEndReached;


    /**
     * Utility/Debug funtion.
     */
    private void printMessage(String comment, SQLException e) {

        SimpleDateFormat formatter
            = new SimpleDateFormat("dd MMM yyyy hh:mm:ss");
        System.out.println(""+formatter.format(new java.util.Date())
            +" "+comment+": "+e.toString());
        lastError = "SQL Exception: "+e.toString();
    }


    public String getLastError() { return lastError; }



    public SelectTableModel(ConnectionServer cs,
            String database, String selectStmt) {
        modelValid = init(cs, database, selectStmt, 0, -1, false, 500);
    }
    public SelectTableModel(ConnectionServer dbs, String database,
            String selectStmt, int maxCols, boolean mu, int cs) {
        modelValid = init(dbs, database, selectStmt, 0, maxCols, mu, cs);
    }




    public void setConnectionServer(ConnectionServer m) {
        dbServer = m;
    }




    /**
     * The init() routine sets up the member variables and calls
     * readFromDatabase().
     */

    private boolean init(ConnectionServer dbs, String database,
            String selectStmt, int ro, int maxCols, boolean mu, int cs) {
        setConnectionServer(dbs);
        manualUpdate = mu;
        cacheSize = cs;
        maxNumberCols = maxCols;
        databaseName = database;
        rowsStored = new SlidingArray(cacheSize);
        selectStatement = selectStmt;
        knownMaxRow = 0;
        dataEndReached = false;
        return readFromDatabase(ro, cacheSize, false);
    }




    /**
     * Get a Statement object from the ConnectionServer used by the table.
     * @throws SQLException re-thrown from ConnectionServer::getStatement().
     */

    private Statement getStatement() throws SQLException {
        Statement rc = null;
        if (dbServer != null) {
            rc = dbServer.getStatement();
        }
        return rc;
    }




    /**
     * Executes a select from the database and fills the cache with
     * row data starting (if possible) at the row offset (ro) specified.
     * The Statement is closed at the end of the method.
     */

    public boolean readFromDatabase(int ro, int numToRead,
            boolean emptyCacheBeforeRead) {
        boolean rc = true;
        ResultSet rs;
        Statement stmt = null;
        // set the model to valid initially - this is important.
        // the false return code of this function will invalidate the model
        // later if the model turns out to be false.
        modelValid = true;
        try {
            stmt = getStatement();
            if (stmt != null) {
                stmt.setCursorName("reader");
                // should combine the next two executes().
                // stmt.execute("use "+databaseName);
                boolean results = stmt.execute(selectStatement);
                // System.out.println(selectStatement);
                if (!results) {
                    // System.out.println("No data for this query");
                    rc = false;
                } else {
                    rs = stmt.getResultSet();
                    rc = copyColumns(rs.getMetaData());
                    if (!readRowSet(ro , rs, numToRead,
                        emptyCacheBeforeRead)) {
                    // System.out.println("Failed to read row sets");
                    }
                }
            } else {
                rc = false;
            }
        } catch (SQLException e) {
            printMessage("Error reading from database", e);
            rc = false;
        } finally {
            if (stmt != null) {
                try {
                    dbServer.releaseStatement(stmt);
                } catch (SQLException e) {
                    printMessage("Error closing statement", e);
                    rc = false;
                } finally {
                    stmt = null;
                }
            }
        }
        return rc;
    }




    /**
     * Empties the datachase and forces a re-read of data from the database.
     */

    public boolean update() {
        boolean rc = readFromDatabase(rowsStored.startIndex(),
            cacheSize, true);
        fireTableDataChanged();
        return rc;
    }




    /**
     * Used to determine if the object was initialized correctly.
     */
    public boolean isValid() {
        return modelValid;
    }




    private boolean copyColumns(ResultSetMetaData rmd) {
        boolean rc = true;
        int cols;
        if (columns != null ) {
            return rc;
        }
        // only performed if columns member is null.
        try {
            if (maxNumberCols == -1 || rmd.getColumnCount() < maxNumberCols) {
                cols = rmd.getColumnCount();
            } else {
                cols = maxNumberCols;
            }
            columns = new ColumnEntry[cols];
            int i;
            for (i=0; i < cols; i++) {
                ColumnEntry e = null;
                e = new ColumnEntry(
                    rmd.getColumnName(i+1),
                    rmd.getColumnType(i+1),
                    rmd.getColumnDisplaySize(i+1));
                columns[i] = e;
            }
        } catch (SQLException e) {
            printMessage("Error reading metadata", e);
            rc = false;
        }
        return rc;
    }




    public int getColumnWidth(int column) {
        if (isValid() && column < columns.length) {
            return columns[column].getWidth();
        } else {
            return 0;
        }
    }

    public String getColumnName(int column) {
        if (isValid() && column < columns.length) {
            return columns[column].getName();
        } else {
            return null;
        }
    }

    public int getColumnType(int column) {
        if (isValid() && column < columns.length) {
            return columns[column].getType();
        } else {
            return 0;
        }
    }

    public int getColumnCount() {
        if (isValid()) {
            return columns.length;
        } else {
            return 0;
        }
    }

    public int getRowCount() {
        int rc = 0;
        if (isValid()) {
            if (dataEndReached) {
                rc = knownMaxRow;
            } else {
                rc = knownMaxRow+rowNumberOverEstimate;
            }
        }
        return rc;
    }




    /**
     * Every time the knownMaxRows changes, 
     * a table change event is fired, allowing the connected viewers to update.
     */
    void setKnownMaxRow(int r) {
        knownMaxRow = r;
        fireTableDataChanged();
    }




    /**
     * Extract a cell value from the model, in addition,
     * if the row requested approaches the boundary of the cache,
     * the cache slides to read in more data.
     * note: row numbering starts at 0.
     */

    public Object getValueAt(int r, int c) {
        String row[] = null;
        Object cell = null;
        row = (String[])rowsStored.get(r);
        if (row != null) {
            if (c < row.length) {
                cell = (Object)row[c];
            }
        }
        if (manualUpdate == false) {
            // check if row requested approaches the boundary, in which case
            // we need to slide the cache along and read new data.
            if (r < rowsStored.startIndex()+dataReadWarning
                && rowsStored.startIndex() > 0) {
                // we are approaching the lower end of the cache and the
                // cache does not current store the lowest possible rows.
                int readFrom  = rowsStored.startIndex()-cacheMoveSize;
                if (readFrom < 0) readFrom = 0;
                readFromDatabase(readFrom, cacheMoveSize, false);
            } else if ((r > rowsStored.endIndex()-dataReadWarning)
                && (!dataEndReached || rowsStored.endIndex() < knownMaxRow-1)) {
                // we are approaching the upper end of the cache.
                int readFrom = rowsStored.endIndex()+1;
                readFromDatabase(readFrom, cacheMoveSize, false);
            }
        }
        return cell;
    }




    /**
     * Fills the data cache from the row set given, starting (if possible)
     * at the given offset.
     */

    private boolean readRowSet(int start, ResultSet rs,
            int numToRead, boolean emptyCacheBeforeRead) {
        boolean moreDataToRead = true;
        int rowOffset = 0;
        // determine if we are moving forwards or backwards in the
        // database relative to what is stored in our cache.
        boolean prepending = false;

        if (emptyCacheBeforeRead) {
            rowsStored.empty();
        }
        if (start < rowsStored.startIndex()) {
            prepending = true;
        }

        // when prepending, need to first read out the result set and
        // then add it to the cache in reverse order.
        String buffer[][] = null;
        int bufferSize = 0;
        if (prepending) {
            buffer = new String[numToRead][];
        }
        while ((rowOffset-start) < numToRead) {
            try {
                moreDataToRead = rs.next();
                rowOffset++;
            } catch (SQLException e) {
                moreDataToRead = false;
                // HANDLE THIS EXCEPTION!
                // printMessage(e);
            }
            if (moreDataToRead == false) {
                // no more data to read.
                break;
            }
            if (rowOffset > start) {
                String[] thisRow = new String[getColumnCount()];
                for(int i = 0; i < getColumnCount(); i++) {
                    if (moreDataToRead) {
                        try {
                            thisRow[i] = rs.getString(i+1);
                        } catch (SQLException e) {
                            thisRow[i] = "";
                        }
                    } else {
                        thisRow[i] = "";
                    }
                } // end of for loop.
                // are we prepending or appending the row?
                if (prepending) {
                    buffer[rowOffset-start-1] = thisRow;
                    bufferSize++;
                } else {
                    rowsStored.append(thisRow);
                }
            } // end of if statement.
            if (prepending && rowOffset >= rowsStored.startIndex()) {
                break;
            }
        } // end of while loop.

        // if prepending - move the data from the buffer to the cache.
        if (prepending) {
            while (bufferSize > 0) {
                bufferSize--;
                rowsStored.prepend(buffer[bufferSize]);
            }
        }

        if (moreDataToRead) {
            if (rowOffset-1 > knownMaxRow) {
                if (!prepending) dataEndReached = false;
                setKnownMaxRow(rowOffset-1);
            }
        } else {
            dataEndReached = true;
            setKnownMaxRow(rowOffset-1);
        }
        return moreDataToRead;
    } // end of fillDataCacheToRow().




    /**
     * Cells are not editable.
     */

    public boolean isCellEditable(int row, int col) {
        return false;
    }


    /**
     * Cells are not editable.
     */

    public void setValueAt(Object value, int row, int col) {
        return;
    }




    /**
     * An inner class for recording column information from the
     * ResultSetMetaData.
     */
    private class ColumnEntry {
        int type;
        int width;
        String name;

        ColumnEntry(String n, int t, int w) {
            name = n;
            type = t;
            width = w;
        }
    
        public int getType() { return type; }
        public int getWidth() { return width; }
        public String getName() { return name; }
    }

} // end of class SelectTableModel.

// end of file SelectTableModel.
