/**
 * These classes have been developed for the DFSLog system.
 *
 * @author N. Kornweibel nkornwei@eso.org
 *
 * TO DO:
 * need @revision, and SNIFF info here.
 * 
 */
package org.eso.dfslog;

import javax.swing.*;
import javax.swing.tree.*;
import java.io.*;


/**
 * TreeGraphReader:
 *
 * The TreeGraphReader accepts and DefaultMutableTreeNode and a filename
 * and reads the file in, creating the tree from the file contents.
 * The file is an ASCII file, containing the one tree node/leaf per line.
 * The tree in the file is described in a depth first manner.
 * Each line has three sections:
 *
 *    - the depth of the node/leaf indicated by the number of '*' characters.
 *    - the name of the node/leaf.
 *    - the leaf command to be executed.
 * 
 * Each line is assumed to be a leaf. If the line following it is indented
 * by one '*', then the previous line becomes a node, with the next line
 * comprising its first leaf.
 * 
 * It is invalid to indent more than one '*' from the previous line.
 * A node can not have its command executed - it is ignored.
 * The name and command info are stored in ViewInfo objects.
 * Lines starting with a '#' character as treated as comment lines and
 * ignored.
 * The following is an example of a tree graph in a file:
 *

   # This is an example file...
   * "This is the first node" "This is the command"
   ** "This is the first leaf of the first node" "Leaf command"
   ** "This is the second leaf of the first node...it is a node"
   *** "First leaf of the node" "command"
   * "This is the first leaf of the root node." "leaf command"
   * "This is the second leaf of the root node." "leaf command"

 */

public class TreeGraphReader {

    public static boolean readTreeGraph(DefaultMutableTreeNode root,
        String fPath) {
        BufferedReader in = null;
        boolean rc = true;

        if (root == null) {
            return false;
        }
        in = openFile(fPath);
        if (in == null) {
            return false;
        }

        DefaultMutableTreeNode currentNode = root;
        try {
            String s;
            int lineNum = 0;
            while ((s = in.readLine()) != null ) {
                // is it a comment line
                lineNum++;
                try {
                    if (s.charAt(0) == '#') {
                        // comment line ...get next line.
                        continue;
                    }
                } catch (StringIndexOutOfBoundsException e) {
                    // empty string...get next line.
                    continue;
                }

                // what is its depth
                int newDepth = calculateDepth(s);
                if (newDepth <= 0 || newDepth > currentNode.getLevel()+1) {
                    System.out.println("Tree Graph error in line: "+lineNum);
                    System.out.println("Invalid tree depth: "+newDepth);
                    continue;
                }

                // create ViewInfo object
                int nameStart = s.indexOf("\"", 0);
                if (nameStart < 0) {
                    System.out.println("Tree Graph error in line: "+lineNum);
                    System.out.println("Failed to find start of name field.");
                    continue;
                }
                int nameEnd = s.indexOf("\"", nameStart+1);
                if (nameEnd < 0) {
                    System.out.println("Tree Graph error in line: "+lineNum);
                    System.out.println("Failed to find end of name field.");
                    continue;
                }
                int commandStart = s.indexOf("\"", nameEnd+1);
                int commandEnd = s.indexOf("\"", commandStart+1);

                ViewInfo vi = null;
                try {
                    if ((commandStart < 0)||(commandEnd < 0)) {
                        vi = new ViewInfo(s.substring(nameStart+1,nameEnd), "");
                    } else {
                        vi = new ViewInfo(s.substring(nameStart+1,nameEnd),
                        s.substring(commandStart+1, commandEnd));
                    }
                } catch (StringIndexOutOfBoundsException e) {
                    vi = null;
                }
                if (vi == null) {
                    System.out.println("Tree Graph error in line: "+lineNum);
                    System.out.println("Failed to collect name/command info");
                    continue;
                }

                // find appropriate node.
                while (currentNode.getLevel() > newDepth-1) {
                    currentNode = 
                        (DefaultMutableTreeNode)currentNode.getParent();
                }
                // create node and attach to appropriate node.
                DefaultMutableTreeNode n = new DefaultMutableTreeNode(vi);
                currentNode.add(n);
                n.setParent(currentNode);
                currentNode = n;
            }
        }
        catch (IOException ex) {
            System.out.println("Error reading file "+fPath+" : "+ex);
        }

        return rc;
    }



    private static int calculateDepth(String s) {
        int i = 0;
        while (i < s.length()) {
            try {
                if (s.charAt(i) == '*') {
                    i++;
                } else {
                    return i;
                }
            } catch (StringIndexOutOfBoundsException e) {
                return i;
            }
        }
        return i;
    }




    private static BufferedReader openFile(String fName) {
        FileInputStream fi = null;
        InputStreamReader ir = null;
        BufferedReader rc = null;
        try {
            fi = new FileInputStream(fName);
        }
        catch (FileNotFoundException ex) {
             return rc;
        }
        ir = new InputStreamReader(fi);
        rc = new BufferedReader(ir);
        return rc;
    }

} // end of class TreeGraphReader.

// end of file TreeGraphReader.java.
