package org.eso.ohs.core.dbb.server;

// $Id: DbbSqlKey.java,v 1.2 2004/04/08 12:09:51 tcanavan Exp $

import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;

/** 
 * This class defines the behavior of a foreign key object, storing
 * the information related to the "start" database table and the
 * "target" database table, in order to build sql joins.<br>
 * It extends Object redefining equals in order to make it a suitable
 * class of objects to be stored into stacks or vectors.
 *
 * @author Carlo Boarotto, ESO 11-Sep-1999
 * 
 **/
public class DbbSqlKey extends Object {
private static final String rcsid = "$Id: DbbSqlKey.java,v 1.2 2004/04/08 12:09:51 tcanavan Exp $";
private static Logger stdlog_ = 
    Logger.getLogger( DbbSqlKey.class );

    /** Key name, should be unique within the database */  
    String name_ = null;

    /** From (primary key) table: first table linked in the join */
    DbbSqlTable pkTable_ = null;

    /** From (primary key) table column linked in the join */  
    String pkColumn_ = null;

    /** To (foreign key) table: second table linked in the join */
    DbbSqlTable fkTable_ = null;

    /** To (foreign key) table column linked in the join */  
    String fkColumn_ = null;

    /**
     * Boolean flag to indicate that the key has been defined by the
     * end application and the path defined by the key has to be
     * preferred when building the join.
     */
    private boolean isPreferredKeyFlag = false;

    /** Preferred starting table used when building the join path */
    private DbbSqlTable preferredStartTable_ = null;

    /** Preferred ending table used when building the join path */
    private DbbSqlTable preferredEndTable_ = null;


    //----------------------------------------
    // Constructors
    //----------------------------------------

    /** 
     * Generate a DbbSqlKey from a ResultSet (produced by
     * DatabaseMetaData.getImportedKeys or
     * DatabaseMetaData.getExportedKeys).
     *
     * @param rs	ResultSet
     * @exception SQLException JDBC exception
     */
    public DbbSqlKey(ResultSet rs)
    	throws SQLException {
    
    	String from_cat = rs.getString(1);
    	String from_sch = rs.getString(2);
    	String from_tbl = rs.getString(3);
    
    	pkTable_ = new DbbSqlTable(from_cat, from_sch, from_tbl);
    	pkColumn_ = rs.getString(4);
    
    	String to_cat = rs.getString(5);
    	String to_sch = rs.getString(6);
    	String to_tbl = rs.getString(7);
    
    	fkTable_ = new DbbSqlTable(to_cat, to_sch, to_tbl);
    	fkColumn_ = rs.getString(8);
    
    	name_ = rs.getString(12);


    }


    /**
     * Constructor called by the end application to build a preferred
     * key. 
     *
     * @param key_name	Key name (should be unique).
     * @param from_tbl	DbbSqlTable indicating the primary key table.
     * @param from_col	String indicating the primary key table column.
     * @param to_tbl	DbbSqlTable indicating the foreign key table.
     * @param to_col	String indicating the foreign key table column.
     * @param pref	Boolean to indicate that the key has to be 
     *			"preferred" when building the join.
     */
    public DbbSqlKey(String key_name, 
		     DbbSqlTable from_tbl,
		     String from_col,
		     DbbSqlTable to_tbl,
		     String to_col) {

    	pkTable_ = from_tbl;
    	pkColumn_ = from_col;
        
    	fkTable_ = to_tbl;
    	fkColumn_ = to_col;
    
    	/* Create a unique key name. */
    	name_ = generateKeyName(from_tbl, from_col, to_tbl, to_col);
    
    	isPreferredKeyFlag = false;
    }

    //----------------------------------------
    // Private methods
    //----------------------------------------

    /**
     * @return A unique key name, dynamically created from the tables and
     * the columns.
     * 
     * @param from_tbl	DbbSqlTable indicating the primary key table.
     * @param from_col	String indicating the primary key table column.
     * @param to_tbl	DbbSqlTable indicating the foreign key table.
     * @param to_col	String indicating the foreign key table column.
     */
    private static String generateKeyName(DbbSqlTable from_tbl,
					  String from_col,
					  DbbSqlTable to_tbl,
					  String to_col) {

	String key_name = null;

	/* Complete column names. */
	String col1 = from_tbl.get() + "." + from_col;
	String col2 = to_tbl.get() + "." + to_col;

	/* Sort column strings in order to have the same name if a key is
	 * referring to the same columns. 
	 */
	if(col1.compareTo(col2) == -1)
	    key_name = "fk" + "_" + col2 + "_" + col1;
	else
	    key_name = "fk" + "_" + col1 + "_" + col2;

	return key_name;
    }


    /** @return from_col_ */
    private String getPkColumn() { return pkColumn_; }


    /** @return to_col_ */
    private String getFkColumn() { return fkColumn_; }
    
    /**  
     * Return a string representation of the sql join identified by
     * the object.
     *
     * @return Sql join.  
     */
    private String getJoinClause() {
        String ret = pkTable_ + "." + pkColumn_ + " = " +
            fkTable_ + "." + fkColumn_;
        return ret;
    }

    /**  
     * Return a string representation of the direct sql join
     * identified by the object.
     *
     * @return Sql join.  */
    private String getDirectJoin() {
        //  String ret = pkTable_ + "." + pkColumn_ + " = " +
        //      fkTable_ + "." + fkColumn_;
        //  return ret;
        return getJoinClause();
    }

    /**  
     * Return a string representation of the reverse sql join
     * identified by the object.
     *
     * @return Sql join.  
     */
    private String getReverseJoin() {
        String ret = fkTable_ + "." + fkColumn_ + " = " +
            pkTable_ + "." + pkColumn_;
        return ret;
    }
    
    //----------------------------------------
    // Public methods
    //----------------------------------------

    /** @return name_ */
    public String getName() { return name_; }

    /** @return from_tbl_ */
    public DbbSqlTable getPkTable() { return pkTable_; }


    /** @return to_tbl_ */
    public DbbSqlTable getFkTable() { return fkTable_; }


    /**
     * Set the preferred path to be used when building the sql join.
     *
     * @param from Database table at the beginning of the join path
     * @param to   Database table at the end of the join path
     */
    public void setPreferredKeyFlag(DbbSqlTable from, 
				    DbbSqlTable to) {

    	if(from == null || to == null)
    	    return;
    	isPreferredKeyFlag = true;
    	preferredEndTable_ = to;
    	preferredStartTable_ = from;
    }

    /**
     * @return A boolean value indicating if the current key object has
     * been defined by the end application and should be preferred when
     * building the joins.  
     */
    public boolean isPreferredKey() { return isPreferredKeyFlag; }

    /**
     * @return A boolean value indicating if the current key object has
     * been defined by the end application and should be preferred when
     * building the joins.  
     */
    public boolean isPreferredKey(DbbSqlTable from, DbbSqlTable to) {
    	if(preferredStartTable_ == null || preferredEndTable_ == null)
    	    return false;
    	if(preferredStartTable_.equals(from) && preferredEndTable_.equals(to))
    	    return isPreferredKeyFlag; 
    	return false;
    }

    /**
     * Redefine Object's method equals in order to make it a suitable
     * class of objects to be stored into stacks or vectors.<br>
     * Two DbbSqlKey are equals if they have the same name (the name of
     * a key should be unique).
     *
     * @param obj	Object
     * @return Boolean value: true if it's equal, false otherwise.  
     */
    public boolean equals(Object obj) {

    	DbbSqlKey key = (DbbSqlKey)obj;
    	String name = key.getName();
    
    	if(name_.equals(name))
    	    return true;
    
    	return false;
    }
    
    /**
     * @return the array of join clauses represented by this key
     */
    public String[] getJoinClauses() {
        String rc[] = new String[2];
        rc[0] = getDirectJoin();
        rc[1] = getReverseJoin();
        //String rc[] = new String[1];
        //rc[0] = getDirectJoin();
        return rc;
    }
    

    /** 
     * @return a string representation of the object.
     */
    public String toString() {
    	String ret = "Foreign key: " + name_ + "\n";
    	ret =  ret + "PK TABLE:    " + pkTable_ + "\n";
    	ret =  ret + "FK TABLE:    " + fkTable_ + "\n";
    	return ret;
    }
}
