/* $Id: DocView.java,v 1.2 2004/07/17 16:45:20 janwunah Exp $  */

package org.eso.ohs.core.docview.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.text.JTextComponent;

import org.eso.ohs.core.docview.datatrans.AdaptorFactory;
import org.eso.ohs.core.docview.datatrans.DocViewObject;
import org.eso.ohs.core.docview.datatrans.ObjectAdaptor;

/**
 * Central class for the DocView package.
 * DocView is a collection of classes that links the values displayed in
 * GUI objects to a property of a Java object.  Thus when the value of the
 * property changes in the object, the value displayed in the GUI component
 * is updated accordingly.  Likewise, if the user changes the value in the
 * GUI component (eg a JTextField or JComboBox), then the corresponding
 * property of the Java object will be updated.
 * <p>
 *
 * <pre>
 *    --------                     ---------                  -----------
 *   |        |   _____________   |         |   __________   |           |
 *   | Java   |--|ObjectAdaptor|--| DocView |--|GuiAdaptor|--| Swing/AWT |
 *   | object |   -------------   | object  |   ----------   | component |
 *   |        |                   |         |                |           |
 *    --------                     ---------                  -----------
 * </pre>
 *
 * <p>
 * The DocView object binds everthing together.  It is the DocView
 * object that is added to the panel where the GUI component is
 * displayed (it subclasses JPanel and contains the component).
 * DocView also listens for <code>PropertyChangeEvent</code>
 * events from the object and <code>FocusGained</code>/
 * <code>FocusLost</code> events from the GUI component.
 * When the object's state changes, the GUI component is always
 * updated.  On the other hand, the object is only updated from the
 * component when the component loses focus and only then if it's
 * state changed while the component had focus.  Using focus in this way
 * allows us to bypass the possibility of infinite recursion due
 * one update triggering another.  It is also a little more efficient
 * to save the GUI changes only when component loses focus, eg the
 * user changes focus to edit some other field.
 *
 * <p>
 * ObjectAdaptor and GuiAdaptor are both interfaces and DocView itself
 * works only with these interfaces.  However, all implementations of
 * ObjectAdaptor are currently just subclasses of ObjAdaptor.  ObjAdaptor
 * itself also contains a factory method that returns an ObjectAdaptor
 * for most of the Java primitive types but applications should use
 * methods of AdaptorFactory, since these can be extended by an
 * application to seamlessly handle additional, application-specific
 * mappings.
 *
 * <p>
 * In order to support updates between the GUI component and
 * a Java object both must fire events when they are modified.
 * Each Java Swing component has it's own event of set of events
 * used for signalling that it's state has changed.  These are listened
 * for by the GuiAdaptor, which sets the IsModified flag appropriately.
 * The Java object must fire a <code>PropertyChangeEvent</code>
 * when it's state is updated, in fact the system assumes that it
 * implements the <code>DocViewObject</code> interface which guarantees
 * that certain methods exist, eg for adding/removing listeners.
 *
 * <p>
 * Typical usage:
 * <pre>
 *      DocView dv = new DocView(new JTextField());
 *      DocViewObject obj = ...
 *                      // Add DocView to panel, not GUI component
 *      myPanel.add(dv, BorderLayout.NORTH);
 *                      // Associate an object and property with the DocView
 *      dv.setObjectProperty(obj, "Property");
 *                      // obj must have public methods
 *                      //      public void setProperty(Type newValue)
 *                      //      public Type getProperty()
 *                      // where Type is a supported property type,
 *                      //      eg int, long, String
 * </pre>
 *
 * <p>
 * @author Bob Kemp
 * @version $Revision: 1.2 $
 * <p>
 * @see AdaptorFactory
 * @see DocViewObject
 * @see ObjectAdaptor
 * @see GuiAdaptor
 */
public class DocView extends JPanel
    implements FocusListener, PropertyChangeListener {

    public static final String cvsID_ =
        "$Id: DocView.java,v 1.2 2004/07/17 16:45:20 janwunah Exp $";

    /**
     * Adaptor for the GUI component that is synchronized
     * with the object's property.
     */
    private GuiAdaptor guiAdaptor_;

    private JComponent comp_;

    private boolean readWrite_ = true;

    /**
     * Adaptor for the Java object with the property
     * that is to be synchronized with the GUI component.
     */
    private ObjectAdaptor objectAdaptor_;

    /** Java object with the property to be updated.  */
    private DocViewObject object_;

    /** Name of the Java object's property to be updated.  */
    private String property_;

    private boolean badValueInGui_;


    /**
     * Helper constructor for JTextField, JTextComponent, etc
     *
     * @param tc        the GUI component
     */
    public DocView(JTextComponent tc) {
        this(new TextCompAdaptor(tc), tc);
    }


    /**
     * Helper constructor for JTextField, JTextComponent, etc
     *
     * @param tc        the GUI component
     */
    public DocView(JTextArea tc) {
        this(new TextCompAdaptor(tc), tc);
        tc.setLineWrap(true);
    }

    /**
     * Helper constructor for JCheckBox.
     *
     * @param ckbox     the GUI component
     */
    public DocView(JCheckBox ckbox) {
        this(new CheckBoxAdaptor(ckbox), ckbox);
    }

    /**
     * Helper constructor for JSlider.
     *
     * @param slider    the GUI component
     */
    public DocView(JSlider slider) {
        this(new SliderAdaptor(slider), slider);
    }

    /**
     * Helper constructor for JComboBox.
     *
     * @param combobox  the GUI component
     */
    public DocView(JComboBox combobox) {
        this(new ComboBoxAdaptor(combobox), combobox);
    }


    /**
     * Constructor for general GUI component
     *
     * @param guiAdaptor        Adaptor for comp
     * @param comp              GUI component
     */
    public DocView(GuiAdaptor guiAdaptor, JComponent comp) {
        guiAdaptor_ = guiAdaptor;
        comp_ = comp;
        setLayout(new BorderLayout());
        add(comp, BorderLayout.CENTER);
        comp.addFocusListener(this);
    }

    /**
     * Return swing component that docview uses.
     */

    public JComponent getComponent() {
        return comp_;
    }


    /**
     * Returns the GUI adaptor for this DocView.
     *
     * @return GUI Adaptor
     */
    public GuiAdaptor getGuiAdaptor() {
        return guiAdaptor_;
    }


    public ObjectAdaptor getObjectAdaptor() {
        return objectAdaptor_;
    }

    /**
     * Sets the Java object and property that is to be linked to
     * the GUI components value.
     *
     * <p>
     * Note: If <code>property</code> = "XYZ" then
     * <code>object</code> must have methods
     * <pre>
     *          public void setXYZ(Type newValue)
     *          public Type getXYZ()
     * </pre>
     * where Type is a supported property type, such as int, long, String.
     * If it does not have these methods or they are not public, an
     * IllegalArgumentException is thrown.
     *
     * @param object    Java object having a property <code>property</code>
     * @param property  Name of a javabeans property of <code>object</code>
     *
     * @exception IllegalArgumentException
     *          thrown if there is no such property in the class.
     */
    public void setObjectProperty(DocViewObject object, String property) {

        if (object_ != null) {
            object_.removePropertyChangeListener(this);
        }
        String oldProperty = property_;
        Object oldObject = object_;

        object_     = object;
        property_   = property;

        if (object_ == null || property == null) {
            objectAdaptor_ = null;
        } else {
            object_.addPropertyChangeListener(this);

            // Special case most common usage
            // Normally we just change the object to another instance
            // of the same class using the same property.
            if (objectAdaptor_ != null &&
                    oldObject != null &&
                    property.equals(oldProperty) &&
                    object.getClass().equals(oldObject.getClass())) {
                objectAdaptor_.setObject(object_);
            } else {
                objectAdaptor_ = AdaptorFactory.getAdaptor(
                    object, property, readWrite_);
            }

            guiAdaptor_.setValue(objectAdaptor_.getValue());
            guiAdaptor_.setModified(false);
            markGuiValid(true);
        }

    }

    /**
     * set background of this object and the GUI component it contains.
     *
     * @param color     hte new color
     */
    public void setBackground(Color color) {
        super.setBackground(color);
        if (guiAdaptor_ != null) {
            comp_.setBackground(color);
        }
    }

    /**
     * Enables or disables this object and the GUI component it contains.
     *
     * @param enabled   true if component is to be enabled, false otherwise
     */
    public void setEnabled(boolean enabled) {
        super.setEnabled(enabled);

        if (guiAdaptor_ != null) {
            guiAdaptor_.setEnabled(enabled);
        }
        readWrite_ = false;
        // XXX Does this blur two concepts???
        // yes it does!!!
        if (objectAdaptor_ != null) {
            objectAdaptor_.setReadWrite(enabled);
        }
    }

    public void setGuiEnabled(boolean enabled) {
        super.setEnabled(enabled);
        if (guiAdaptor_ != null) {
            guiAdaptor_.setEnabled(enabled);
        }
        readWrite_ = false;
    }

    public void setObjAdaptorEnabled(boolean enabled) {
        if (objectAdaptor_ != null) {
            objectAdaptor_.setReadWrite(enabled);
        }
    }

    /**
     * FocusGained event handler.
     *
     * @param ev        Event object containing more details
     */
    public void focusGained(FocusEvent ev) {
        // stdlog_.debug("Focus Gained");
        // On Solaris there is an extra focus gained event been generated
        // on a combobox in certain situations. This means an editied field
        // is set back to modified false without a focusLost event happenning
        // and an update occuring. Since we cannot rely on Swing to handle this properly
        // we need to comment this out here to insure updates. This does not seem to
        // make the system less efficent so lets get rid of the setModified here.
        // See bug desc. in bugzilla
        //
        //guiAdaptor_.setModified(false);
    }

    /**
     * FocusLost event handler.
     *
     * @param ev        Event object containing more details
     */
    public void focusLost(FocusEvent ev) {
        // stdlog_.debug("Focus Lost");
        updateObjectFromGui();
    }

    private boolean shouldUpdateObject() {
        return objectAdaptor_ != null && guiAdaptor_.isModified();
    }

    private boolean shouldUpdateGui() {
        return ! badValueInGui_;
    }

    public void updateObjectFromGui() {
        if (shouldUpdateObject()) {
            try {
                guiAdaptor_.setModified(false);
                String obj = guiAdaptor_.getValue();
               /*
                * put a baseline we don't want to have identical as far as the
                * the values are concerned but different when it comes to trailing
                * non printable characters (item 1238 and item 1121
                */
                if(obj != null){
					obj = obj.trim(); // baseline
					objectAdaptor_.setValue(obj);
					markGuiValid(true);
                }
            } catch (IllegalArgumentException e) {
                markGuiValid(false);
            }
        }
    }

    private void markGuiValid(boolean isValid) {
        badValueInGui_ = !isValid;
        if (guiAdaptor_ != null) {
            guiAdaptor_.setForeground(isValid ? Color.black : Color.red);
        }
    }

    public void addNotify() {
        guiAdaptor_.setModified(false);
        super.addNotify();
    }

    public void removeNotify() {
        super.removeNotify();
        if (shouldUpdateObject()) {
            SwingUtilities.invokeLater(
                new Runnable() {
                    public void run() {
                        updateObjectFromGui();
                    }
                }
            );
        }
    }

    /**
     * PropertyChangeEvent event handler.
     *
     * @param ev        Event object containing more details
     */
    public void propertyChange(PropertyChangeEvent ev) {
        // stdlog_.debug("Object Changed");
        if (objectAdaptor_ != null && shouldUpdateGui()) {
            guiAdaptor_.setValue(objectAdaptor_.getValue());
        }
    }
}

