package org.eso.ohs.core.utilities;

/**
 * A base implementation of the GTreeNode interface.
 *
 * Please see GTreeModel for other details.
 *
 * @author Nick Kornweibel for ESO.
 */

import java.util.Enumeration;
import java.util.Hashtable;


public class DefaultGTreeNode implements GTreeNode {

    protected Hashtable children_ = new Hashtable();
    protected GTreeNode parent_ = null;
    protected String name_ = null;
    protected Object value_ = null;

    /**
     * Initialize the tree node with the same name and value
     */
    public DefaultGTreeNode(String name) {
        name_ = name;
        value_ = name;
    }

    /**
     * Initialize the tree node with the name and val
     */
    public DefaultGTreeNode(String name, Object val) {
        name_ = name;
        value_ = val;
    }

    /**
     * @return the path to this node (goes recursively up ancestors).
     */
    public String getPathName() {
	StringBuffer path = new StringBuffer();
	if (!parent_.isRoot()) {
	    path.append(getPathParentName());
	}
	path.append(getName());
        return path.toString();
    }
 /**
     * @return the path to this node (goes recursively up ancestors).
     */
    public String getPathParentName() {
	StringBuffer path = new StringBuffer();
	if (!parent_.isRoot()) {
	    path.append(parent_.getPathParentName());
	    path.append(parent_.getName());
	    path.append(".");

	}
        return path.toString();
    }

    /**
     * @return the name of this node.
     */
    public String getName() {
        return name_;
    }

    /**
     * Set the name of the node
     */
    public void setName(String name) {
        name_ = name;
    }

    /**
     * @return the value object of this node.
     * Useful when a node value object is not a String.
     */
    public Object get() {
        return value_;
    }

    /**
     * Set the value object
     */
    public void set(Object val) {
        value_ = val;
    }

    /**
     * @return true if obj is a GTreeNode and has the same name as this.
     */
    public boolean equals(Object obj) {
        boolean rc = false;
        if (obj != null && obj instanceof GTreeNode) {
            GTreeNode node = (GTreeNode)obj;
            rc = getName().equals(node.getName());
        }
        return rc;
    }

    /**
     * @return the value of value object toString() method.
     * Used in the majority of cases as the value is a String.
     */
    public String getValue() {
        String rc = null;
        if (value_ != null) {
           rc = value_.toString();
        }
        return rc;
    }

    /**
     * Set the value object as a String
     */
    public void setValue(String val) {
        value_ = val;
    }

    /**
     * Set the parent of this to node.
     */
    public void setParent(GTreeNode node) {
        parent_ = node;
    }

    /**
     * @return the parent of this node, or null if there is not one.
     */
    public GTreeNode getParent() {
        return parent_;
    }

    /**
     * @return true if node is a leaf, else false.
     * If the node has no children, it is a leaf.
     */
    public boolean isLeaf() {
        return (getChildCount() == 0);
    }
  /**

     * @return true if node is the root, else false.
     *
     */
   public boolean isRoot() {
       return (parent_ == null);
   }

    /**
     * @return the child of node with the name childName, or null if not found.
     */
    public GTreeNode getChild(String childName) {
        return (GTreeNode)children_.get(childName);
    }

    /**
     * @return the child of node with the name child.getName(),
     * or null if not found.
     */
    public GTreeNode getChild(GTreeNode child) {
        return (GTreeNode)children_.get(child.getName());
    }

    /**
     * @return the number of children of node.
     */
    public int getChildCount() {
        return children_.size();
    }

    /**
     * @return an Enumeration of node's children.
     */
    public Enumeration getChildren() {
        return children_.elements();
    }

    /**
     * Add child as a child or aNode (no checking for duplicates).
     * The childs parent is set to this.
     */
    public void addChild(GTreeNode child) {
        child.setParent(this);
        children_.put(child.getName(), child);
    }

    /**
     * @return (and remove) child with name child.getName(),
     * or null if not found.
     * Child's parent is set to null.
     */
    public GTreeNode removeChild(GTreeNode child) {
        return removeChild(child.getName());
    }

    /**
     * @return (and remove) the first child with the name childName.
     * Child's parent is set to null.
     */
    public GTreeNode removeChild(String childName) {
        GTreeNode rc = (GTreeNode)children_.remove(childName);
        if (rc != null) {
            rc.setParent(null);
        }
        return rc;
    }
}
