package org.eso.ohs.evm;

/**
 * Specialization of the EVMExecutor that uses the OHSConfig data to 
 * initialize drivers and script directories.
 *
 * This class also implements the EVMScriptFinder, used to locate scripts
 * for execution.
 *
 * This class initializes based on keywords in the OHSConfig configuration.
 * It loads all drivers specified in the configuration as well as executing
 * all init scripts and using the timeout specified in the configuration.
 *
 * This class uses a singleton pattern. However the getInstance() method
 * will return null if EVM is not configured (to differentiate
 * it from configured with initialization errors).
 *
 * @author Nick Kornweibel
 */

import java.io.File;
import java.io.FilenameFilter;
import java.util.Enumeration;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;

public class EvmExecutorImpl extends EVMExecutor {

    /**
     * Logging class.
     */
    protected static Logger stdlog_ =
        Logger.getLogger(EvmExecutorImpl.class);


    /**
     * Singleton instance of EVMExecutor - will remain null if
     * the EVM module is not configured
     */

    private static EvmExecutorImpl instance_;

    /**
     * Set to false if there is no config data for the Evm module
     */

    private static boolean evmConfigured_ = true;

    /**
     * Config keyword (base) for script interpreters:
     *     SCRIPTING.DRIVER.1, SCRIPTING.DRIVER.2, etc.
     */

    public static final String driverKeyword = "SCRIPTING.DRIVER.";
   
    /**
     * Config keyword (base) for setup scripts:
     *     SCRIPTING.SETUP.1, SCRIPTING.SETUP.2, etc.
     */

    public static final String initScriptKeyword = "SCRIPTING.SETUP.";

    /**
     * Config keyword for timeout value
     */

    public static final String timeoutKeyword = "SCRIPTING.TIMEOUT";

    /** 
     * Script prefix
     */

    public static final String evmScriptPrefix = "evm";
    
    /** 
      * Script prefix
      */

    public static final String execScriptPrefix = "exec";

    

    /** 
     * EVM Directory (under instrument directory)
     */

    public static final String evmDir = "evm";
    
    
    /**
      * EVM Directory (under instrument directory)
      */
    public static final String execDir = "exectime";

    /**
     * Constructor - private, called through getInstance()
     */

    private EvmExecutorImpl(String[] scriptDrivers) {
        super(scriptDrivers, new P2PPScriptFinder());
    }

    public static boolean evmConfigured() {
        setup();
        return evmConfigured_;
    }

    /**
     * Get an instance of this EVMExecutor
     * @return the singlton instance of this class
     */

    public static EvmExecutorImpl getInstance() {
        setup();
        return instance_;
    }

    private static void setup() {
        if (instance_ == null && evmConfigured_) {
            OHSConfig config = OHSConfig.getInstance();
            String[] scriptDrivers = extractDrivers(config);
    
            if (scriptDrivers.length > 0) {
    
                String[] initScripts = extractScripts(config);
                long timeout = extractTimeout(config);
    
                instance_ = new EvmExecutorImpl(scriptDrivers);
    
                if (instance_.isInitialized()) {
                    if (timeout > 0L) {
                        instance_.setScriptTimeout(timeout*1000L);
                    }
                    try {
                        for (int i = 0; i < initScripts.length; i++) {
                            String script = initScripts[i];
                            instance_.runScript(new File(script));
                        }
                    } catch (EVMException ex) {
                        // failed to run init scripts...disable the EVM
                        stdlog_.debug("EVM Init Script execution error: "
                                +ex.getMessage());
                        instance_.disable("Init script execution error: "
                                +ex.getMessage());
                    }
                } else {
                    stdlog_.debug("EVM Initialition error: "
                            +instance_.getStatusString());
                }
            } else {
                // EVM was not configured for this app
                evmConfigured_ = false;
            }
        } else {
            // already setup
        }
    }

    /**
     * Extract the init script info from the config info
     * @param the config data
     * @return set of intitialization scripts
     */

    private static String[] extractScripts(OHSConfig config) {
        Vector scripts = new Vector();
        int i = 1;
        do {
            String script = config.getValue(initScriptKeyword+i);    
            if (script.length() == 0) {
                // no more scripts...
                break;
            } else {
                scripts.addElement(script);
            }
            i++;
        } while (true);
 
        // copy elements from Vector to return array 
        String[] rc = new String[scripts.size()];
        for (i = 0; i < rc.length; i++) {
            rc[i] = (String)scripts.elementAt(i);
        }
        return rc;
    }

    /**
     * Extract the scripting driver info from the config info
     * @param the config data
     * @return set of configured drivers
     */

    private static String[] extractDrivers(OHSConfig config) {
        Vector drivers = new Vector();
        int i = 1;
        do {
            String driver = config.getValue(driverKeyword+i);
            if (driver.length() == 0) {
                // no more drivers...
                break;
            } else {
                drivers.addElement(driver);
            }
            i++;
        } while (true);
 
        // copy elements from Vector to return array 
        String[] rc = new String[drivers.size()];
        for (i = 0; i < rc.length; i++) {
            rc[i] = (String)drivers.elementAt(i);
        }
        return rc;
    }

    /**
     * Extract the timeout value (in seconds) from the config data
     * @param the config data
     * @return the timeout in seconds, or -1L if not found
     */

    private static long extractTimeout(OHSConfig config) {
        long rc = -1L;

        String timeout = config.getValue(timeoutKeyword);
        if (timeout.length() != 0) {
            try {
                rc = Long.parseLong(timeout);
            } catch (NumberFormatException ex) {
                // junk!
                rc = -1L;
            }
        }
        return rc;
    }

    /**
     * A basic interface to verify an OB - returns error, warning and info
     * messages to the passed StringBuffer params.
     * @param the observation block to be verified
     * @param string buffer for info messages
     * @param string buffer for warning messages
     * @param string buffer for error messages
     * @param string buffer for header messages
     * @param string buffer for footer messages
     * NOTE: This call must be synchronized in order to make the EvmExecutor
     * and the script interpreters thread-safe.
     */

    public synchronized void verify(CalibrationBlock ob, StringBuffer info,
            StringBuffer warn, StringBuffer error,
            StringBuffer header, StringBuffer footer) {
        if (isInitialized()) {
            P2PPEvmMessageListener msgCollector 
                = new P2PPEvmMessageListener();
            addEVMMessageListener(msgCollector);
            try {
                CalibrationBlock[] obs = new CalibrationBlock[1];
                obs[0] = ob;
                verify(obs);
                info.append(msgCollector.getMessages());
                warn.append(msgCollector.getWarnings());
                error.append(msgCollector.getErrors());
            } catch (EVMException ex) {
                info.append(msgCollector.getMessages());
                warn.append(msgCollector.getWarnings());
                error.append(msgCollector.getErrors()+"\n");
                error.append("Exception in verification: "+ex.getMessage());
            }
            String hMsg = msgCollector.getHeader();
            if (hMsg != null) {
                header.append(hMsg);
            }
            String fMsg = msgCollector.getFooter();
            if (fMsg != null) {
                footer.append(fMsg);
            }
            removeEVMMessageListener(msgCollector);
        }
    }

    /**
     * A basic interface to verify an OB - returns if verification
     * passed or failed and report, header and footer
     * messages to the passed StringBuffer params.
     * Report messages are returned: header, msgs (in order received), footer.
     * <br>ChangeLog: cmavros 21-12-01 added
     * @param the observation block to be verified
     * @param string buffer where the report is returned.
     * @param string buffer where the header is returned.
     * @param string buffer where the footer is returned.
     * @return boolean of the passing status of the verification 
     * NOTE: This call must be synchronized in order to make the EvmExecutor
     * and the script interpreters thread-safe.
     */

    public synchronized boolean verify(CalibrationBlock ob,
           StringBuffer report, StringBuffer header, StringBuffer footer) {

        StringBuffer error = new StringBuffer();
        boolean verifyPassed = true;
        if (isInitialized()) {
            P2PPEvmMessageListener msgCollector
                = new P2PPEvmMessageListener();
            addEVMMessageListener(msgCollector);
            try {
                CalibrationBlock[] obs = new CalibrationBlock[1];
                obs[0] = ob;
                verify(obs);

                String hMsg = msgCollector.getHeader();
                if (hMsg != null) {
                    header.append(hMsg);
                }
                report.append(msgCollector.getAllMessages());
                String fMsg = msgCollector.getFooter();
                if (fMsg != null) {
                    footer.append(fMsg);
                }
                error.append(msgCollector.getErrors());
                if ((error.toString().trim()).length() > 0){
                    verifyPassed = false;
                }
            } catch (EVMException ex) {
                String hMsg = msgCollector.getHeader();
                if (hMsg != null) {
                    header.append(hMsg);
                }
                report.append(msgCollector.getAllMessages());
                report.append("Exception in verification: "+ex.getMessage());
                error.append(msgCollector.getErrors());
                error.append("Exception in verification: "+ex.getMessage());
                String fMsg = msgCollector.getFooter();
                if (fMsg != null) {
                    footer.append(fMsg);
                }
                return false;
            }
            removeEVMMessageListener(msgCollector);
        }
        return verifyPassed;
    }

    /**
     * A basic interface to verify an OB - returns 
     * if the verification passed or failed.
     * Report messages are returned: header, msgs (in order received), footer.
     * <br>ChangeLog: cmavros 21-Dec-01 added
     * @param the observation block to be verified
     * @return boolean of the passing status of the verification 
     * NOTE: This call must be synchronized in order to make the EvmExecutor
     * and the script interpreters thread-safe.
     */

    public synchronized boolean verify(CalibrationBlock ob) {
        StringBuffer error = new StringBuffer();
        boolean verifyPassed = true;
        if (isInitialized()) {
            P2PPEvmMessageListener msgCollector
                = new P2PPEvmMessageListener();
            addEVMMessageListener(msgCollector);
            try {
                CalibrationBlock[] obs = new CalibrationBlock[1];
                obs[0] = ob;
                verify(obs);
                error.append(msgCollector.getErrors());
                if ((error.toString().trim()).length() > 0){
                    verifyPassed = false;
                }
            } catch (EVMException ex) {
                error.append(msgCollector.getErrors());
                error.append("Exception in verification: "+ex.getMessage());
                return false;
            }
            removeEVMMessageListener(msgCollector);
        }
        return verifyPassed;
    }
    /**
        * A basic interface to verify an OB - returns 
        * if the verification passed or failed.
        * Report messages are returned: header, msgs (in order received), footer.
        * <br>ChangeLog: cmavros 21-Dec-01 added
        * @param the observation block to be verified
        * @return boolean of the passing status of the verification 
        * NOTE: This call must be synchronized in order to make the EvmExecutor
        * and the script interpreters thread-safe.
        */

       public synchronized void calculateETCS(CalibrationBlock ob) throws EVMException {
           StringBuffer error = new StringBuffer();
           boolean verifyPassed = true;
           if (isInitialized()) {
               P2PPEvmMessageListener msgCollector
                   = new P2PPEvmMessageListener();
               addEVMMessageListener(msgCollector);
               try {
                   CalibrationBlock[] obs = new CalibrationBlock[1];
                   obs[0] = ob;
                   calculateETCS(obs);
                   error.append(msgCollector.getErrors());
                   if ((error.toString().trim()).length() > 0){
                       verifyPassed = false;
                   }
               } catch (EVMException ex) {
                   ex.printStackTrace();
                   throw new EVMException(ex.getMessage());
                   //error.append(msgCollector.getErrors());
                   //error.append("Exception in Cal: "+ex.getMessage());                  
               }
               removeEVMMessageListener(msgCollector);
           }           
       }


}


/**
 * Script finder class implementation for P2PP - uses the static
 * settings in P2PPEvmExecutor class and the data in the OB to 
 * extract scripts from the instrument package in use.
 */

class P2PPScriptFinder implements EVMScriptFinder {

    /**
     * Logging class.
     */
    protected static Logger stdlog_ =
        Logger.getLogger(P2PPScriptFinder.class);

    /**
     * Implementation of the EVMScriptFinder interface. The directory
     * is read each time in oder to include any new scripts.
     */

    private FilenameFilter evmFileFilter_ = new FilenameFilter() {
            public boolean accept(File dir, String name) {
                EvmExecutorImpl pe = EvmExecutorImpl.getInstance();
                String fileExtensions[] = pe.getSupportedScriptExtensions();
                File f = new File(dir.getAbsolutePath(), name);
                boolean rc = name.startsWith(EvmExecutorImpl.evmScriptPrefix)
                        && f.isFile();
                if (rc) {
                    int i = 0;
                    while (i < fileExtensions.length) {
                        if (name.endsWith("."+fileExtensions[i])) {
                            break;
                        }
                        i++;
                    }
                    rc = (i != fileExtensions.length);
                }
                return rc;
            }
        };
        
        
    /**
        * Implementation of the EVMScriptFinder interface. The directory
        * is read each time in oder to include any new scripts.
        */

       private FilenameFilter execFileFilter_ = new FilenameFilter() {
               public boolean accept(File dir, String name) {
                   EvmExecutorImpl pe = EvmExecutorImpl.getInstance();
                   String fileExtensions[] = pe.getSupportedScriptExtensions();
                   File f = new File(dir.getAbsolutePath(), name);
                   boolean rc = name.startsWith(EvmExecutorImpl.execScriptPrefix)
                           && f.isFile();
                   if (rc) {
                       int i = 0;
                       while (i < fileExtensions.length) {
                           if (name.endsWith("."+fileExtensions[i])) {
                               break;
                           }
                           i++;
                       }
                       rc = (i != fileExtensions.length);
                   }
                   return rc;
               }
           };


    public Vector findScriptsEVM(CalibrationBlock ob) {
        Vector rc = new Vector();
        String instrumentName = ob.getOd().getInstrument();
        float version = ob.fetchIPVersionFromOd();
        Instrument inst = InstrumentList.getInstance().getInstrument(instrumentName, version);
        if (inst != null) {
            String instrumentDir = inst.getTSFDirectory();
            File evmDir = new File(
                    instrumentDir+File.separator+File.separator
                    +EvmExecutorImpl.evmDir);
            stdlog_.debug("EVM: Searching for scripts in: "+evmDir.toString());
            if (evmDir.isDirectory()) {
                String[] files = TextUtils.sort(evmDir.list(evmFileFilter_));
                for (int i = 0; i < files.length; i++) {
                    rc.addElement(new File(evmDir.getAbsolutePath()
                            +File.separator+files[i]));
                }
            }
        } else {
            stdlog_.debug("EVM: Could not locate Instrument for this object.");
        }
        return rc;
    }
    
    public Vector findScriptsExecTime(CalibrationBlock ob) {
           Vector rc = new Vector();
           String instrumentName = ob.getOd().getInstrument();
           float version = ob.fetchIPVersionFromOd();
           Instrument inst = InstrumentList.getInstance().getInstrument(instrumentName, version);
           if (inst != null) {
               String instrumentDir = inst.getTSFDirectory();
               File execDir = new File(
                       instrumentDir+File.separator+File.separator
                       +EvmExecutorImpl.execDir);
               stdlog_.debug("EVM: Searching for scripts in: "+ execDir.toString());
               if (execDir.isDirectory()) {
                   String[] files = TextUtils.sort(execDir.list(execFileFilter_));
                   for (int i = 0; i < files.length; i++) {
                       rc.addElement(new File(execDir.getAbsolutePath()
                               +File.separator+files[i]));
                   }
               }
           } else {
               stdlog_.debug("EVM: Could not locate Instrument for this object.");
           }
           if(rc.size() ==0) {
               stdlog_.debug("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ERROR");
           }             
           return rc;
           
    }

}


/**
 * EVMMessageListener implementation that simply gathers all
 * messages into a single String
 */

class P2PPEvmMessageListener implements EVMMessageListener {

    /**
     * records all error, warning and info messages in the order received
     */
    private Vector allMessages_ = new Vector();

    /**
     * header messages - null indicates the default header is to be printed.
     */
    private Vector headers_ = null;

    /**
     * footer messages - null indicates the default footer is to be printed.
     */
    private Vector footers_ = null;

    /**
     * Called when a script sends an information message
     * @param the message
     */

    public void infoMessage(String msg) {
        allMessages_.addElement(new InfoMessage(msg+"\n"));
    }

    /**
     * Called when a script sends a warning message
     * @param the message
     */

    public void warningMessage(String msg) {
        allMessages_.addElement(new WarningMessage("warning: "+msg+"\n"));
    }

    /**
     * Called when a script sends an error message
     * @param the message
     */

    public void errorMessage(String msg) {
        allMessages_.addElement(new ErrorMessage("error: "+msg+"\n"));
    }

    /**
     * Called when a script adds a message to the report header
     * The first call to this function will replace the default
     * header.
     * @param message to be added to the report header
     */

    public void headerMessage(String msg) {
        if (headers_ == null) {
            headers_ = new Vector();
        }
        headers_.addElement(new InfoMessage(msg+"\n"));
    }

    /**
     * Called when a script adds a message to the report footer
     * The first call to this function will replace the default
     * footer.
     * @param message to be added to the report footer
     */

    public void footerMessage(String msg) {
        if (footers_ == null) {
            footers_ = new Vector();
        }
        footers_.addElement(new InfoMessage(msg+"\n"));
    }

    /**
     * clear previously reported errors
     */

    public void errorsWereCleared() {
        Vector msgs = new Vector();
        for (Enumeration e = allMessages_.elements(); e.hasMoreElements(); ) {
            Message m = (Message)e.nextElement();
            if (!(m instanceof ErrorMessage)) {
                msgs.addElement(m);
            }
        }
        allMessages_ = msgs;
    }

    /**
     * clear previously reported warnings
     */

    public void warningsWereCleared() {
        Vector msgs = new Vector();
        for (Enumeration e = allMessages_.elements(); e.hasMoreElements(); ) {
            Message m = (Message)e.nextElement();
            if (!(m instanceof WarningMessage)) {
                msgs.addElement(m);
            }
        }
        allMessages_ = msgs;
    }

    /**
     * Get the messages collected so far
     * @return string of collected messages
     */

    public String getMessages() {
        StringBuffer msgs = new StringBuffer();
        for (Enumeration e = allMessages_.elements(); e.hasMoreElements(); ) {
            Message m = (Message)e.nextElement();
            if (m instanceof InfoMessage) {
                msgs.append(m.toString());
            }
        }
        return msgs.toString();
    }

    /**
     * Get the messages collected so far
     * @return string of collected messages
     */

    public String getWarnings() {
        StringBuffer msgs = new StringBuffer();
        for (Enumeration e = allMessages_.elements(); e.hasMoreElements(); ) {
            Message m = (Message)e.nextElement();
            if (m instanceof WarningMessage) {
                msgs.append(m.toString());
            }
        }
        return msgs.toString();
    }

    /**
     * Get the messages collected so far
     * @return string of collected messages
     */

    public String getErrors() {
        StringBuffer msgs = new StringBuffer();
        for (Enumeration e = allMessages_.elements(); e.hasMoreElements(); ) {
            Message m = (Message)e.nextElement();
            if (m instanceof ErrorMessage) {
                msgs.append(m.toString());
            }
        }
        return msgs.toString();
    }

    /**
     * Get the header messages.
     * @return the header messages. Null indicates no headers msgs were sent.
     */

    public String getHeader() {
        String rc = null;
        if (headers_ != null) {
            StringBuffer msgs = new StringBuffer();
            for (Enumeration e = headers_.elements(); e.hasMoreElements(); ) {
                Message m = (Message)e.nextElement();
                msgs.append(m.toString());
            }
            rc = msgs.toString();
        }
        return rc;
    }

    /**
     * Get the footer messages.
     * @return the footer messages. Null indicates no footer msgs were sent.
     */

    public String getFooter() {
        String rc = null;
        if (footers_ != null) {
            StringBuffer msgs = new StringBuffer();
            for (Enumeration e = footers_.elements(); e.hasMoreElements(); ) {
                Message m = (Message)e.nextElement();
                msgs.append(m.toString());
            }
            rc = msgs.toString();
        }
        return rc;
    }

    /**
     * @return all error, warning, info messages in the order received
     */

    public String getAllMessages() {
        StringBuffer msgs = new StringBuffer();
        for (Enumeration e = allMessages_.elements(); e.hasMoreElements(); ) {
            Message m = (Message)e.nextElement();
            msgs.append(m.toString());
        }
        return msgs.toString();
    }

    /**
     * Internal interface for a Message
     */

    public static interface Message {
        public String toString();
    }

    /**
     * Internal class for Info Message - also used for storing
     * header and footer messages.
     */

    public static class InfoMessage implements Message {
        private String msg_;
        protected InfoMessage(String msg) {
            msg_ = msg;
        }
        public String toString() {
            return msg_;
        }
    }

    /**
     * Internal class for Warning Message
     */

    public static class WarningMessage implements Message {
        private String msg_;
        protected WarningMessage(String msg) {
            msg_ = msg;
        }
        public String toString() {
            return msg_;
        }
    }

    /**
     * Internal class for Error Message
     */

    public static class ErrorMessage implements Message {
        private String msg_;
        protected ErrorMessage(String msg) {
            msg_ = msg;
        }
        public String toString() {
            return msg_;
        }
    }
}
