package org.eso.ohs.instruments;

import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.log4j.Logger;

/**
 * Instanciates a paramater class of type integer. Verifies string input
 * in the GUI to make sure a valid integer within a specified range is entered.
 * <p>
 * $Id: IntegerParameter.java,v 1.10 2004/04/08 12:09:52 tcanavan Exp $
 * @author Tim Canavan 
 */

public class IntegerParameter extends Parameter {

    static final long serialVersionUID = 9091241679416306593L;

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( IntegerParameter.class );


    /**
     * Minimum value an integer paramter must have
     */ 
  
    private Integer minValue_;

    /**
     * Maximum value the integer value must not exceed
     */

    private Integer maxValue_;

    /**
     * List of valid integers that the integer values
     * that the parameter value can have.
     */

    private Integer validValues_[];


   /**
    * Costructor for IntegerParameter
    * @param parameterName name of parameter
    */


    public IntegerParameter(ParamMetaData pMeta) {
        super(pMeta);
        initValidValues();        
    }    

    /**
     * Minumum value that the parameter value must be
     * @return minValue
     */


    public Integer getMinValue() {
        return minValue_;
    }

    /**
     * Maximum value that the parameter value should not exceed
     * @return maximum value
     */

    public Integer getMaxValue() {
        return maxValue_;
    }
 
    /**
     * List of valid values that the parameter can be assigned to
     * @return list of valid values
     */

    public Integer[] getValidValues() {
        return validValues_;
    }


    public char getDBTypeCode() {
        return 'I';
    }

    /**
     * Verify a valid integer has been entered.
     * @return If value is integer and in the 
     *         range defined by the paramater class return OK    
     *         The range of valid values
     */
  
    public boolean verifyValue(String value) {
        Integer ivalue = null;
        String defaultValue = getDefaultValue();
        if(value == null || value.length() <=0) {             
	    return false;
        }          
        if(value.equals("NODEFAULT")) {
             return false;
        }
        try {
            ivalue = new Integer(value);
        }
        catch(NumberFormatException e) {
	  //stdlog_.debug("Invalid integer value " + e.getMessage());                 
            return false;               
	}
        String range = getRange();
        if(range != null && range.length() > 0) {
             if(!valueInRange(ivalue)) {
                 return false;
             }      
        }
        return true; 
    }

    /**
     * Returns the appropiate ToolTip text for the parameter
     * using the prameters range and validValues 
     * @return Tooltip text
     */

    public String getToolTipText() {
        boolean range=false;
        String tooltp  = "Enter valid value";
        if (minValue_ != null
        &&  maxValue_ != null)
            range = true;     
        StringBuffer validIntegers = new StringBuffer(20); 
        if(validValues_ != null) {
	   for(int i=0;i<validValues_.length;i++)
               validIntegers.append(validValues_[i].intValue() + " ");
        }
        StringBuffer rangeBuffer = null;
        if(range) {
             rangeBuffer = new StringBuffer();
             rangeBuffer.append(getMinValue());
             rangeBuffer.append("..");
             rangeBuffer.append(getMaxValue());
	}
        if(range && validValues_ == null)
            tooltp = "Integer within range " + rangeBuffer.toString();
        if(!range && validValues_ != null)
            tooltp = "Integer value from the list " + validIntegers.toString();
        if(range && validValues_ != null)
           tooltp = "Integer value within " + rangeBuffer.toString() + " or in list "
                                        + validIntegers.toString();
        if(!range && validValues_ == null)
            tooltp = "Enter integer value";
       return tooltp;
    }

    /**
     * Check to see if an integer value is in the specified range
     * A range of 1 2..4000 3 means that any values between 2 and 400
     * are valid along with 1 and 3.
     * @return boolean 
     *
     */


    boolean valueInRange(Integer value) {                               
        if(maxValue_ != null && minValue_ != null) {           
            if(value.intValue() <=  maxValue_.intValue() &&
            value.intValue() >= minValue_.intValue())
                return true;            
        }
        if(validValues_ != null) {
            for(int i=0;i<validValues_.length;i++) {
                if(value.intValue() == validValues_[i].intValue())
                    return true;
            }
        }
        return false;          
    }

   /**
     * Check to see if an integer value is in the specified range
     * A range of 1 2..4000 3 means that any values between 2 and 400
     * are valid along with 1 and 3.
     * @return boolean 
     *
     */


    public void initValidValues() {
        String range = getRange();
        if(range == null)
            return;
        StringTokenizer token = new StringTokenizer(range);
        StringBuffer minValueStr = new StringBuffer(20);
        StringBuffer maxValueStr = new StringBuffer(20);             
        Vector validValues = new Vector();
        boolean valid = false;
        int j=0;
        while(token.hasMoreElements()) {
            String rangeItem  = token.nextToken();
            int index = rangeItem.indexOf("..");
            if(index != -1) {                   
                int i = index-1;
                while(i >= 0 && rangeItem.charAt(i) != ' ') {
                    minValueStr.insert(0,rangeItem.charAt(i));
                    i--;
                }
                i = index+2;
                while(i < rangeItem.length() && rangeItem.charAt(i) != ' ') {
                    maxValueStr.append(rangeItem.charAt(i));
                    i++;
                }         
                try {
                    minValue_ = new Integer(minValueStr.toString());
                    maxValue_ = new Integer(maxValueStr.toString());                   
                }
                catch(NumberFormatException e) {
                          stdlog_.debug("Invalid range in paramater " + getParameterName());                         
                }                                  
            }
            else {
		try {
                    Integer validValue = new Integer(rangeItem);
                    validValues.addElement(validValue);                   
                }
                catch(NumberFormatException e) {
                    stdlog_.debug("Invalid range in paramater " + getParameterName());
                }
                    
            }               
         }        
         if(validValues.size() > 0) {
             validValues_ = new Integer[validValues.size()];
             validValues.copyInto(validValues_);
         }
    }


    /**
     * This method is for debugging purposes only
     * @return Debugging string 
     */

    public String toString() {
        String s = super.toString();
        StringBuffer buf = new StringBuffer();
        buf.append(s);
        if(minValue_ != null)
            buf.append("Min value " + minValue_.toString() + "\n");
        if(maxValue_ != null)
            buf.append("MaxValue  " + maxValue_.toString() + "\n");
        if(validValues_ != null && validValues_.length > 0) {
           for(int i=0;i<validValues_.length;i++)
               buf.append("Valid values " + validValues_[i].toString());
	}
        return buf.toString();

    } 

}


