package org.eso.ohs.instruments;

import java.io.IOException;
import java.io.Serializable;

import org.eso.ohs.core.utilities.AssocList;
import org.eso.ohs.core.utilities.TextUtils;

/**
 * This is an  base class that stores generic information for all 
 * parameter types.
 * <p>
 *
 * @author Tim Canavan 
 */

public abstract class Parameter implements Serializable {

	// For Serialization 

	static final long serialVersionUID = -4378250031958131854L;

	private final String rcsid =
		"$Id: Parameter.java,v 1.31 2005/02/07 09:06:36 pnunes Exp $";

	public static final String FILTER_NAME = "Filter";
	public static final int FILTER_VALUE = 1;

	public static final String GRISM_NAME = "Grism";
	public static final int GRISM_VALUE = 2;

	public static final AssocList filters =
		new AssocList().add(FILTER_NAME, new Integer(FILTER_VALUE)).add(
			GRISM_NAME,
			new Integer(GRISM_VALUE));

	public static final int PARAM_DISPLAY_VALUE_MAXLEN = 300;

	/**
	 * Actual value of the parameter
	 */
	private String value_;

	/**
	 * Name of the parameter 
	 */

	private String parameterName_;

	/** 
	 * The type of parameter e.g. number,numlist
	 */

	private String parameterType_;

	/**
	 * Refernce to read only meta Data required for compatibility
	 * with Version 2.1.x caches
	 */

	private ParamMetaData pMeta_;

	/**
	 * New transient refernce to Meta Data class so all this information
	 * is not stoted to disk as in Version 2.1.x
	 */

	private transient ParamMetaData newPMeta_;

	/**
	 * Constructs an instance of the parmater class
	 * @param pMeta Paramaters Meta Data 
	 */

	public Parameter(ParamMetaData pMeta) {
		newPMeta_ = pMeta;
		parameterName_ = newPMeta_.getParameterName();
		parameterType_ = newPMeta_.getType();
		value_ = newPMeta_.getValue();
	}

	/**
	 * Read parameter details from local storage
	 * @param in Stream to capture data from cache
	 * @exception IOException IO Exception if record cannot be read from disk
	 * @exception ClassNotFoundException class not found exception
	 */

	private void readObject(java.io.ObjectInputStream in)
		throws IOException, ClassNotFoundException {
		in.defaultReadObject();

		// for backwards compatibility with 2.1.X caches

		if (pMeta_ != null) {
			parameterType_ = pMeta_.getType();
			parameterName_ = pMeta_.getParameterName();
			if (value_ == null)
				value_ = pMeta_.getValue();
			pMeta_ = null;
		}
	}

	/**
	 * Check to see if a parameter keyword is valid
	 * @param paramKeyword parameter Keyword DET.DIT etc..
	 * @return Valid or invalid  
	 */

	public static boolean validParamKeyword(String paramKeyword) {
		return ParamMetaData.validParamKeyword(paramKeyword);
	}

	public boolean displayValueDifferent() {
		return false;
	}

	/**     
	 * Should the parameter be displayed on the GUI or not
	 * @return Boolean value to see if a value should be displayed
	 */
	public boolean isDisplay() {
		return newPMeta_.isDisplay();
	}

	/**
	 * Value is Hidden or not. Unlike isDisplay the means that 
	 * a parameter can be hidden to some applications and not others.
	 * See instrument documentation
	 * @return Boolean value
	 */

	public boolean isHidden() {
		return newPMeta_.isHidden();
	}

	/**
	 * Parameter name
	 * @return Name of parameter
	 * @see #parameterName_
	 */
	public String getParameterName() {
		return parameterName_;
	}

	/**
	 * An alias for <code>getParameterName()</code>.
	 * <br>ChangeLog: A M Chavan, 09-Aug-2001
	 * @return Name of parameter
	 * @see #getParameterName()
	 */
	public String getName() {
		return getParameterName();
	}

	public abstract char getDBTypeCode();

	/**
	 * Type of Paramater integer = 'integer'
	 * @return Type = integer = 'I'
	 */

	public String getType() {
		return parameterType_;
	}

	/**
	 * Range of allowed values
	 * @return Range of allowed values
	 */

	public String getRange() {
		return newPMeta_.getRange();
	}

	/**
	 * Initial value of parameter
	 * @return defaultValue_ the defaule setting
	 */
	public String getDefaultValue() {
		return newPMeta_.getDefaultValue();
	}

	/**
	 * @return  Current value of the parameter
	 */
	public String getValue() {
		return value_;
	}

	public void setMetaData(ParamMetaData pMeta) {
		newPMeta_ = pMeta;
		parameterType_ = newPMeta_.getType();
		parameterName_ = newPMeta_.getParameterName();
	}

	/**
	 * @return Label describing the parameter  
	 */
	public String getLabel() {
		return newPMeta_.getLabel();
	}

	/**
	 * @return Mini help comment for parameter  
	 */
	public String getMiniHelp() {
		return newPMeta_.getMiniHelp();
	}

	/**
	 * @return Flag: whether this value should be able to change  
	 */
	public boolean isFixed() {
		return newPMeta_.isFixed();
	}

	public boolean hasHiddenKeyword() {
		return newPMeta_.hasHiddenKeyword();
	}

	/**
	 * setValue sets the value of a parameter. I found that this method is
	 *          called by methods like
	 *                    org.eso.ohs.p2pp.views.AcquisitionModel.setValueAt()
	 *           and      org.eso.ohs.p2pp.views.ODGridModel.setValueAt()
	 *          As this setValueAt methods are not able to provide a default
	 *          value, this setValue method is the appropriate place to replace
	 *          a possible null value with a default value.
	 *
	 * @param value New value of parameter
	 *              may be null if combobox is de-selected
	 *
	 * <br>ChangeLog: Toni Schlichting 28-June-2001
	 */

	public void setValue(String value) {
		if (value == null) {
			value_ = newPMeta_.getDefaultValue();
		} else {
			value_ = value;
		}
	}

	public boolean samePrefix(String compareString) {
		return newPMeta_.samePrefix(compareString);
	}

	// public String verify(String value) {return "OK";}

	public abstract boolean verifyValue(String value);

	// public abstract boolean verify();

	public boolean verify() {
		return verifyValue(getValue());
	}

    /**
     * @return OBD for the provided value.
     */
	private String valueToOBD(String value) {
        String parameterName = getParameterName();
		if (value == null) {
			value = "";
		}
		if (parameterName.indexOf(".FILT") >= 0) {
			// Convert each space to a hash '#'
			value = value.replace(' ', '#');
		} else if (
			parameterName.endsWith(".ALPHA")
				|| parameterName.endsWith(".DELTA")) {
			StringBuffer buff = new StringBuffer();
			int len = value.length();
			for (int k = 0; k < len; k++) {
				char ch = value.charAt(k);
				if (Character.isDigit(ch) || ch == '.' || ch == '-') {
					buff.append(ch);
				}
			}
			value = buff.toString();
		}
		return value;
	}
   
    /**
     * @return an OBDParameterString using the parameter value
     */
    public String toOBDParameterString() {
        return toOBDParameterString(getValue());
    }

    /**
     * @return an OBDParameterString using the value provided
     */
	public String toOBDParameterString(String value) {
		StringBuffer sb = new StringBuffer();
		sb.append(
			parameterName_
				+ "\t\t\""
				+ TextUtils.escapedString(valueToOBD(value))
				+ "\"");
		sb.append("\n");
		return sb.toString();
	}

    /**
     * @return a DisplayOBDParameterString using the parameter value
     */
    public String toDisplayOBDParameterString() {
        return toDisplayOBDParameterString(getValue());
    }

    /**
     * @return a DisplayOBDParameterString using the value provided
     */
	public String toDisplayOBDParameterString(String value) {
		StringBuffer sb = new StringBuffer();
		if (value.length() > PARAM_DISPLAY_VALUE_MAXLEN) {
			String s = value.substring(0, PARAM_DISPLAY_VALUE_MAXLEN);
			sb.append(
				parameterName_
					+ "\t\t\""
					+ TextUtils.escapedString(s)
					+ " ...."
					+ "\"");
		} else {
			sb.append(
				parameterName_
					+ "\t\t\""
					+ TextUtils.escapedString(valueToOBD(value))
					+ "\"");
		}
		sb.append("\n");
		return sb.toString();
	}

	public String getDisplayValue() {
		if (getValue() != null)
			return getValue();
		if (getDefaultValue() != null)
			return getDefaultValue();
		return "NODEFAULT";
	}

	/**
	 * Get tooltip text for a parameter
	 * @return tooltip text for parameter
	 */

	public String getToolTipText() {
		return "Enter Value";
	}

	/**
	 * <br>ChangeLog: A M Chavan, 09-Aug-2001: rewritten
	 */
	public String toString() {

		StringBuffer sb = new StringBuffer();

		sb.append(getClass());
		sb.append(", Name=");
		sb.append(getName());
		sb.append(", Type=");
		sb.append(getType());
		sb.append(", Range=");
		sb.append(getRange());
		sb.append(", Default=");
		sb.append(getDefaultValue());
		sb.append(", DispValue=");
		sb.append(getDisplayValue());
		sb.append(", Hidden=");
		sb.append(isHidden());
		sb.append(", Display=");
		sb.append(isDisplay());
		sb.append(", Fixed=");
		sb.append(isFixed());
		sb.append(", Label=");
		sb.append(getLabel());
		sb.append(", Minihelp=");
		sb.append(getMiniHelp());

		return sb.toString();
	}
	
	/**
	 * Compares two <code>Parameter</code> objects
	 * checking equivalence in <code>parameterName_</code> and <code>value_</code> fields 
	 * 
	 * @param other
	 * @return boolean
	 */
	public boolean equals(Parameter other) {
	    
	    if ((  this.parameterName_.compareTo(other.getParameterName()) == 0) &&
	        ( value_.compareTo(other.getValue()) == 0  )                 )
	        return(true);
	    else
	        return(false);
	        
	} 
}
