package org.eso.ohs.persistence;

import java.util.EventObject;

import org.eso.ohs.dfs.StorableObject;

/**
 * These are the events that are sent to the AutoSaver thread.
 * There is one event for each of the various actions that 
 * are applicable to a storeable object OBJECT_CREATED,
 * OBJECT_READ,OBJECT_REMOVED. 
 *
 * <p>
 * $Id: ObjectEvent.java,v 1.9 2003/07/03 08:49:49 ddorigo Exp $
 * <br>
 * @author Bob Kemp
 */

public class ObjectEvent extends EventObject {

    /**
     * Constant to identify an OBJECT_CREATED event     
     */
    public static final int OBJECT_CREATED = 1;

    /**
     * Constant to identify an OBJECT_READ event     
     */
    public static final int OBJECT_READ    = 2;

    /**
     * Constant to identify an OBJECT_MOVED event     
     */
    public static final int OBJECT_MOVED   = 3;


     /**
     * Constant to identify an OBJECT_FORGOTTEN event     
     */
    public static final int OBJECT_FORGOTTEN   = 6;

    /**

    /**
     * Constant to identify an OBJECT_REMOVED event     
     */
    public static final int OBJECT_REMOVED = 5;

   /**
    * Contains a value of one of the constants defined above
    */
    private int type_;

    /**
     * A reference to the Storable Object which the event is to 
     * take place on.
     */
    private StorableObject sobj_;

    /**
     * Id of the Storable Object which the event is to take place on.
     */
    private long id_;

    /**
     * Type of the Storable Object which the event is to take place on.
     */
    private Class objType_;
   
    /**
     * The destination media where the objects will be saved to
     * be it local storage,RDBMS etc..
     */
    private Media device_;

    /**
     * This specifies where the StorableObject was previously located
     * if this is a OBJECT_MOVED event, but is null otherwise.
     */
    private DirectoryNode fromDir_;

    /**
     * This is the directory node that specifies where the StorableObject
     * will be placed in local storage. This will be unused when 
     * we are reading and writing objects to the database.
     */
    private DirectoryNode toDir_;

    /**
     * Constructor for ObjectEvent
     *
     * @param type	ObjectEvent type
     * @param device	Selected storage media
     * @param dir	Location for storeable object
     * @param id	Id of Storable Object that the event refers to
     * @param objType	Class of Storable Object that the event refers to
     *
     */
    public ObjectEvent(int type, Media device, DirectoryNode fromDir,
			DirectoryNode toDir, StorableObject sobj,
			long id, Class objType) {
	super(new Long(id));

	type_       = type;
	sobj_	    = sobj;
	id_ 	    = id;
	objType_    = objType;
	device_     = device;
	fromDir_    = fromDir;
	toDir_	    = toDir;
    }

    /**
     * Constructor for ObjectEvent
     *
     * @param type ObjectEvent type
     * @param device Selected storage media
     * @param dir Location for storeable object when Media is local storage
     * @param sobj Storable Object that the event refers to
     *
     */
    public ObjectEvent(int type, Media device, DirectoryNode fromDir,
			DirectoryNode toDir, StorableObject sobj) {
	super(new Long(sobj.getId()));

	type_       = type;
	sobj_	    = sobj;
	id_ 	    = sobj.getId();
	objType_    = sobj.getClass();
	device_     = device;
	fromDir_    = fromDir;
	toDir_	    = toDir;
    }

    /**
     * Retrieves ObjectEvent type. 
     * @return Type of event OBJECT_CREATED,OBJECT_REMOVED
     */
  

    public int getType() {
	return type_;
    }

    /**
     * Retrieves media type 
     * @return Device where Storable object will be stored.
     */
  

    public Media getMedia() {
	return device_;
    }

   /**
    * Retrieves the DirectoryNode for the Storable Object
    * @return Location in Local Storage where Storable Object will
    *              be stored.  
    */
    public DirectoryNode getOldDirectoryNode() {
	return fromDir_;
    }

   /**
    * Retrieves the DirectoryNode for the Storable Object
    * @return Location in Local Storage where Storable Object will
    *              be stored.  
    */
    public DirectoryNode getDirectoryNode() {
	return toDir_;
    }

    /**
     * Retrieves the StorableObject linked with the ObjectEvent.
     *
     * @return Reference to the StorableObject that the event relates to.
     */
    public StorableObject getStorableObject() {
	/**********
	if (type_ == OBJECT_REMOVED || type_ == OBJECT_MOVED) {
	    throw new IllegalArgumentException(
		"This should not be called for this type of event"
	    );
	}
	**********/
	return sobj_;
    }

    /**
     * Retrieves the id of the StorableObject linked with the ObjectEvent.
     *
     * @return id of the StorableObject that the event relates to.
     */
    public long getObjectId() {
	return id_;
    }

    /**
     * Retrieves the class of the StorableObject linked with the ObjectEvent.
     *
     * @return class of the StorableObject that the event relates to.
     */
    public Class getObjectClass() {
	return objType_;
    }

}

