package org.eso.ohs.persistence;

import java.util.Enumeration;
import java.util.Hashtable;

import org.apache.log4j.Logger;
import org.eso.ohs.dfs.BusinessObject;


    /**
     * The Registry is a table of cached StorableObject instances.
     * There will only ever be one Registry at any time but it is
     * not a singleton because it is contained within the AbstractObjectManager,
     * which is itself a singleton.
     */
    public  class Registry {
	/**
	 * References to ObjectInfo records for every StorableObject that
	 * is currently in memory.  The table is keyed by the object's id
	 * wrapped in a java.lang.Long.
	 * 
	 * <p>
	 * Using this table, we can ensure that at most one instance of
	 * any StorableObject is kept in memory at any one time.
	 */
	protected final Hashtable inMemory_ = new Hashtable();
 private static Logger stdlog_ = 
	Logger.getLogger( Registry.class );

	/**
	 * Return a RegistryEntry object for the given id, or
	 * null if not found.
	 * 
	 * @param id	id of the StorableObject to be searched for
	 * 
	 * @return	RegistryEntry for the object, or null if not found
	 */
	public RegistryEntry lookup(long id) {
	    return lookup(new Long(id));
	}

	/**
	 * Return a RegistryEntry object for the given id (wrapped in a Long),
	 * or null if not found.
	 * 
	 * @param idObj	id of the StorableObject to be searched for,
	 * 		wrapped in a java.lang.Long
	 * 
	 * @return	RegistryEntry for the object, or null if not found
	 */
	public synchronized RegistryEntry lookup(Long idObj) {
	    	    stdlog_.debug (" ID "+ idObj );
		        stdlog_.debug (" Obj "+ inMemory_.get(idObj) );
	    return (RegistryEntry)inMemory_.get(idObj);
	}


	/**
	 * Return a RegistryEntry object for the given id (wrapped in a Long),
	 * or null if not found.
	 * 
	 * @param idObj	id of the StorableObject to be searched for,
	 * 		wrapped in a java.lang.Long
	 * 
	 * @return	RegistryEntry for the object, or null if not found
	 */
	public synchronized RegistryEntry add(long id, RegistryEntry oi) {
	    if (oi == null) {
		throw new NullPointerException("Bad Registry entry: " + oi);
	    }
	    
	    stdlog_.debug (" look obj "+ id);
	    if (lookup(id) == null) {
		stdlog_.debug (" added obj "+ oi);
		inMemory_.put(new Long(id), oi);
	    } 
	    return lookup(id);
	}


	/**
	 * Remove the table entry for the given id, if it exists.
	 * 
	 * @param id	id of the table entry to be removed
	 * 
	 * @return	RegistryEntry for the object, or null if not found
	 */
	public synchronized RegistryEntry remove(long id) {
	    return (RegistryEntry)inMemory_.remove(new Long(id));
	}


	/**
	 * Returns true if there are objects in the Registry that are
	 * recorded as being in this directory.
	 * 
	 * @param node	DirectoryNode for the directory to be checked
	 * 
	 * @return	true if there are one or more entries for this
	 * 		directory, false otherwise
	 */
	public synchronized boolean hasObjectsInDirectory(DirectoryNode node) {
	    Enumeration enum = inMemory_.keys();
	    RegistryEntry oi;

	    while (enum.hasMoreElements()) {
		oi = lookup((Long)enum.nextElement());
		if (node.equals(oi.getDirectory())
				&& oi.getObject() instanceof BusinessObject) {
			// The directory contains an object that hasn't been
			//  saved to external media yet: abort deletion.
		    return true;
		}
	    }

	    return false;
	}
    }
