package org.eso.ohs.persistence.dbase.phase2;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.instruments.InstrumentPackage;
import org.eso.ohs.instruments.ZipUtils;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;

/**
 * I/O class for IPs stored in the database.
 * <p>
 * @version $Revision: 1.1 $
 * @author Ron J Lobo
 * @author cmavros
 * <br>
 * @see org.eso.ohs.persistence.dbase.phase2.DbaseMgr
 * @see org.eso.ohs.persistence.ObjectManager
 */
public class InstrumentPackageDBIO {

    public static final String cvsID_ =
        "$Id: InstrumentPackageDBIO.java,v 1.1 2005/03/30 09:56:12 tcanavan Exp $";

    /** Codes used by getIPInternal() */
    private static final int SaveAsFile = 0;     // save zip file to disk
    private static final int SaveAndExtract = 1; // unzip the zip file
    private static final int GetIPobject = 2;    // return InstrumentPackage

    /** Codes used by getObsRuns() */
    public static final float anyVersion = -1.0f;
    public static final float unassignedVersion = -2.0f;

    /** Standard log channel */
    private static final Logger stdlog_ =
        Logger.getLogger(InstrumentPackageDBIO.class);

    private static final String obrepDbName =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());

    private static final String opc70DbName =
        Config.getCfg().getDbName(ObservingRun.getSuffix());


    private static final String IP = obrepDbName + "..instrument_packages";
    private static final String IPP= obrepDbName + "..ip_periods";
    private static final String SR = opc70DbName + "..sched_rep";
    private static final String OR = opc70DbName + "..obs_runs";
    private static final String OP = opc70DbName + "..obs_programmes";
    private static final String PR = opc70DbName + "..proposed";
    private static final String EU = opc70DbName + "..eso_users";
    private static final String RS = obrepDbName + "..run_states";


    /**
     * retrieves a list of instruments from table instrument_packages
     *
     * @return instrument list
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static String[] getInstruments() throws SQLException {

        Connection con = null;
        Statement stmt = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()),
                cfg.getDbUserName(), cfg.getDbPassword());
            stmt = con.createStatement();

            // first find out the size of the zip file
            String sql = "select distinct inst_name from " + IP;
            stdlog_.debug(sql);

            ResultSet rs = stmt.executeQuery(sql);
            Vector instrVec = new Vector();
            while (rs.next()) {
                String inst = (rs.getString("inst_name")+"").trim();
                instrVec.addElement(inst);
                stdlog_.debug("found: " + inst);
            }
            rs.close();
            stmt.close();
            ConnectionPool.getPool().releaseConnection(con);

            String[] instrArr = new String[instrVec.size()];
            instrVec.copyInto(instrArr);
            return instrArr;

        } catch (SQLException e) {
            stdlog_.error("getInstruments(): " + e.getMessage());
            if (con != null) {
                stmt.close();
                ConnectionPool.getPool().releaseConnection(con);
            }
            throw e;
        }
    }


    /**
     * inserts a row in a table
     *
     * @param table     table to add a row to
     * @param columns   columns for which to provide values
     * @param vals      values to add
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static boolean insertRow(String table, String columns, String vals) {

        boolean rtn = true;
        Connection con = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()),
                cfg.getDbUserName(), cfg.getDbPassword());
            String SQLstr = "insert into " + obrepDbName + ".." + table +
                " (" + columns + ") values (" + vals + ")";
            stdlog_.debug(SQLstr);
            Statement stmt = con.createStatement();
            stmt.executeUpdate(SQLstr);
            stmt.close();

        } catch (SQLException e) {
            rtn = false;
            stdlog_.error("Failed to insert: " + e.getMessage());

        } finally {
            if (con != null) {
                ConnectionPool.getPool().releaseConnection(con);
            }
        }
        return rtn;
    }


    /**
     * executes a group of sql commands as a single transaction
     *
     * @param sqlCmds   SQL commands in strings
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static void executeCmds(Vector sqlCmds) throws SQLException {

        Connection con = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()),
                cfg.getDbUserName(), cfg.getDbPassword());
            con.setAutoCommit(false);

            for (int i = 0; i < sqlCmds.size(); i++) {
                String SQLstr = (String)sqlCmds.elementAt(i);
                stdlog_.debug(SQLstr);
                Statement stmt = con.createStatement();
                stmt.executeUpdate(SQLstr);
                stmt.close();
            }

            // all commands succeeded - commit transaction
            con.commit();
            stdlog_.debug("all SQL commands executed");

        } catch (SQLException e) {
            tryRollback(con, e);
            throw e;

        } finally {
            if (con != null) {
                try  {
                    con.setAutoCommit(true);
                    stdlog_.debug("setAutoCommit: true");
                } catch (SQLException ex) {
                    stdlog_.error("Error setAutoCommit(true):" + ex.getMessage());
                }
                ConnectionPool.getPool().releaseConnection(con);
            }
        }
    }


    /**
     * adds an IP by inserting rows in instrument_packages and ip_periods
     *
     * @param instr     name of instrument
     * @param version   IP version
     * @param addedBy   name of inserter
     * @param addDate   time of insertion
     * @param comments  comments
     * @param file_     file obj to add to db
     * @param periods   periods this IP applies to
     *
     * @exception SQLException  an error occurred during the dbase operation
     * @exception FileNotFoundException  the zip file couldn't be found
     */
    public static void insertIP(String instr, float version,
            String addedBy, int addDate,
            String comments, File file_, int[] periods)
            throws SQLException, FileNotFoundException {

        Connection con = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()),
                cfg.getDbUserName(), cfg.getDbPassword());
            con.setAutoCommit(false);

            String SQLstr = "insert into " + IP +
                " (inst_name, version, added_by, added_date, comment, " +
                "inst_package) values ('" +
                instr + "', " + version + ", " +
                SqlUtils.sqlEscapedString(addedBy) + ", " + addDate + ", " +
                SqlUtils.sqlEscapedString(comments) + ", ?)";
            stdlog_.debug(SQLstr);

            java.sql.PreparedStatement ps = con.prepareStatement(SQLstr);
            FileInputStream fis = new FileInputStream(file_);
            ps.setBinaryStream(1, fis, (int)file_.length());

            int res = ps.executeUpdate();
            ps.close();

            // ip_periods table
            for (int i = 0; i < periods.length; i++) {
                SQLstr = "insert into " + IPP +
                    " (inst_name, version, period) values ('" +
                    instr + "', " + version + ", " + periods[i] + ")";
                stdlog_.debug(SQLstr);
                Statement stmt = con.createStatement();
                stmt.executeUpdate(SQLstr);
                stmt.close();
            }


            // all inserts succeeded - commit transaction
            con.commit();
            stdlog_.debug("insertIP: success");

        } catch (SQLException e) {
            tryRollback(con, e);
            throw e;

        } catch (FileNotFoundException e) {
            tryRollback(con, e);
            throw e;

        } finally {
            if (con != null) {
                try  {
                    con.setAutoCommit(true);
                    stdlog_.debug("setAutoCommit: true");
                } catch (SQLException ex) {
                    stdlog_.error("Error setAutoCommit(true):" + ex.getMessage());
                }
                ConnectionPool.getPool().releaseConnection(con);
            }
        }
    }


    /**
     * updates an IP by updating rows in instrument_packages and
     * adding/deleting rows in ip_periods
     *
     * @param instr     name of instrument
     * @param version   IP version
     * @param addedBy   name of inserter
     * @param comments  comments
     * @param periods   periods this IP applies to
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static void updateIP(String instr, float version, String addedBy,
        String comments, int[] periods) throws SQLException {

        Connection con = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()),
                cfg.getDbUserName(), cfg.getDbPassword());
            con.setAutoCommit(false);

            // update instrument_packages
            String SQLstr = "update " + IP +
                " set added_by=" + SqlUtils.sqlEscapedString(addedBy) +
                ", comment=" + SqlUtils.sqlEscapedString(comments) +
                " where inst_name='" + instr + "' and version=" + version;
            stdlog_.debug(SQLstr);
            Statement stmt = con.createStatement();
            stmt.executeUpdate(SQLstr);
            stmt.close();

            if (periods != null) {
                // delete from ip_periods
                SQLstr = "delete from " + IPP +
                    " where inst_name='" + instr + "' and version=" + version;
                stdlog_.debug(SQLstr);
                stmt = con.createStatement();
                stmt.executeUpdate(SQLstr);
                stmt.close();

                // insert into ip_periods
                String values = "'" + instr + "', " + version + ", ";
                for (int i = 0; i < periods.length; i++) {
                    SQLstr = "insert into " + IPP +
                        " (inst_name, version, period) values (" +
                        values + periods[i] + ")";
                    stdlog_.debug(SQLstr);
                    stmt = con.createStatement();
                    stmt.executeUpdate(SQLstr);
                    stmt.close();
                }
            }

            // all commands succeeded - commit transaction
            con.commit();
            stdlog_.debug("updateIP: success");

        } catch (SQLException e) {
            tryRollback(con, e);
            throw e;

        } finally {
            if (con != null) {
                try  {
                    con.setAutoCommit(true);
                    stdlog_.debug("setAutoCommit: true");
                } catch (SQLException ex) {
                    stdlog_.error("Error setAutoCommit(true):" + ex.getMessage());
                }
                ConnectionPool.getPool().releaseConnection(con);
            }
        }
    }


    private static void tryRollback(Connection con, Exception e) {
        stdlog_.error("caught exception: " + e.getMessage());
        try {
            con.rollback();
            stdlog_.debug("rollback done");
        } catch (SQLException ex) {
            stdlog_.error("Error rolling back: " + ex.getMessage());
        }
    }


    /**
     * Deletes an IP from the database
     *
     * @param instr     instrument name
     * @param version   version number
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static boolean deleteIP(String instr, float version) {

        boolean rtn = true;
        stdlog_.debug("delete IP: " + instr + " v" + version);
        Connection con = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()),
                cfg.getDbUserName(), cfg.getDbPassword());
            Statement stmt = con.createStatement();

            String sql = "delete from " + IPP +
                " where inst_name = '" + instr + "' and version = " + version;
            stdlog_.debug(sql);
            int del1 = stmt.executeUpdate(sql);

            sql = "delete from " + IP +
                " where inst_name ='" + instr + "' and version = " + version;
            stdlog_.debug(sql);
            int del2 = stmt.executeUpdate(sql);
            stdlog_.debug("del1=" + del1 + ", del2=" + del2);

            stmt.close();

        } catch (SQLException e) {
            rtn = false;
            stdlog_.error("Failed to delete IP: " +e.getMessage());
        } finally {
            if (con != null) {
                ConnectionPool.getPool().releaseConnection(con);
            }
        }
        return rtn;
    }


    /**
     * Checks to see if a instrument and version already exist in the DB
     *
     * @param instr     instrument name
     * @param version   IP version
     * @return          boolean
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static boolean checkDuplicateIP(String instr, float version)
    throws SQLException {
        Connection con = null;
        Statement stmt = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()),
                cfg.getDbUserName(), cfg.getDbPassword());
            stmt = con.createStatement();
            String sql = "select * from " + IP +
                " where inst_name = '" + instr + "'" +
                " and version = " + version;
            stdlog_.debug(sql);
            ResultSet rs = stmt.executeQuery(sql);
            return (rs.next());
        } finally {
            if (con != null) {
                stmt.close();
                ConnectionPool.getPool().releaseConnection(con);
            }
        }
    }


    /**
     * retrieves a IP given the instrument and version
     *
     * @param   instr   instrument name
     * @param   version IP version
     *
     * @return InstrumentPackage object
     *
     * @exception SQLException  an error occurred during the dbase operation
     * @exception IOException   bad processigCode or error extracting zipfile
     */
    public static InstrumentPackage getIP(String instr, float version)
            throws SQLException, IOException {
        return getIPInternal(instr, version, null, GetIPobject, null);
    }


    /**
     * retrieves a IP and saves to disk, given the instrument and version
     *
     * @param instr     instrument name
     * @param version   IP version
     * @param path      Path to save or extract the file to
     * @param extract   should the IP be unzipped
     * @param conn      The connection to use, if null, a connection
     *                  is requested from the connection pool
     *
     * @return filename of the extracted IP zip file
     *
     * @exception SQLException  an error occurred during the dbase operation
     * @exception IOException   bad processigCode or error extracting zipfile
     */
    public static String getIP(
            String instr, float version, String path, boolean extract)
            throws SQLException, IOException {
        return getIP(instr, version, path, extract, null);
    }

    public static String getIP(
            String instr, float version,
            String path, boolean extract, Connection conn)
            throws SQLException, IOException {
        int pCode = extract ? SaveAndExtract : SaveAsFile;
        InstrumentPackage ip = getIPInternal(instr, version, path, pCode, conn);
        String filename = ip.getComment();
        return filename;
    }


    /**
     * retrieves a IP and returns it as a byte array
     * given the instrument and version
     *
     * @param   instr   instrument name
     * @param   version IP version
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static byte[] getIPData(String instr, float version)
            throws SQLException, IOException {
        InstrumentPackage ip = getIPInternal(instr, version, null, GetIPobject, null);
        return ip.getInstPackageFile();
    }


    /**
     * retrieves a IP and processed the IP data as required
     * by the operationCode, given the instrument and version.
     *
     * @param   instr           instrument name
     * @param   version         IP version
     * @param   processingCode  what to do with the IP:
     *                          SaveAsFile, SaveAndExtract, GetIPobject
     * @param   externCon       The connection to use, if null a connection from
     *                          the connection pool is created.
     *
     * @exception SQLException  an error occurred during the dbase operation
     * @exception IOException   bad processigCode or error extracting zipfile
     */
    protected static InstrumentPackage getIPInternal(
            String instr, float version,
            String path, int processingCode, Connection externCon)
            throws SQLException, IOException {

        InstrumentPackage ip = new InstrumentPackage(instr, version);
        Connection con = externCon;
        Statement stmt = null;

        try {
            Config cfg = Config.getCfg();
            if (con == null) {
                con = ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(), cfg.getDbPassword());
            }
            stmt = con.createStatement();

            // first find out the size of the zip file
            String sql = "select max(datalength(inst_package)) from " + IP +
                " where inst_name = '" + instr + "'" +
                " and version = " + version;
            stdlog_.debug(sql);
            ResultSet rs = stmt.executeQuery(sql);
            int textLenMax = 0;
            while (rs.next()) {
                textLenMax = rs.getInt(1);
            }
            rs.close();

            // set textsize to size of the zip file
            sql = "set textsize " + textLenMax;
            stdlog_.debug(sql);
            stmt.execute(sql);

            // now retrieve the inst_package
            sql = "select added_by, added_date, comment, inst_package from " +
                IP + " where inst_name = '" + instr + "' and version = " + version;
            stdlog_.debug(sql);
            rs = stmt.executeQuery(sql);

            if (rs.next()) {
                switch (processingCode) {
                case SaveAsFile:
                case SaveAndExtract:
                    int addedDate = rs.getInt("added_date");
                    InputStream inStrm = rs.getBinaryStream("inst_package");
                    String v1 = version + "000.000";
                    String v2 = v1.substring(0, v1.indexOf('.')+3);
                    String zipFile = instr + "-" + v2 + "-" +
                        dateString(addedDate) + ".zip";
                    stdlog_.debug("zipFile=" + zipFile);
                    ip.setComment(zipFile); // hack!
                    try {
                        boolean xtract = (processingCode == SaveAndExtract);
                        ZipUtils.extract(inStrm, path, zipFile, xtract);
                    } catch (IOException e) {
                        stdlog_.error("zip-extract error: " + e.getMessage());
                        throw e;
                    }
                    break;

                case GetIPobject:
                    ip.setAddedBy((rs.getString("added_by")+"").trim());
                    ip.setDateAdded(rs.getLong("added_date"));
                    ip.setComment((rs.getString("comment")+"").trim());
                    ip.setInstPackageFile(rs.getBytes("inst_package"));
                    break;

                default:
                    throw new IOException("Unknown processing code: " + processingCode);
                }
            } else {
                throw new SQLException("Instrument Package for " + instr +
                    " v" + version + " not found in the database");
            }

            rs.close();
            stmt.close();
            if (externCon == null) {
                // only release the connection if it was not
                // passed into the method.
                ConnectionPool.getPool().releaseConnection(con);
            }

        } catch (SQLException e) {
            stdlog_.error("getIP(): " + e.getMessage());
            if (con != null) {
                stmt.close();
                if (externCon == null) {
                    // only release the connection if it was not
                    // passed into the method.
                    ConnectionPool.getPool().releaseConnection(con);
                }
            }
            throw e;
        }

        return ip;
    }
    
    
    public static String getInstFromOd(Statement stmt,long od_id) throws SQLException {
        
        String sql = "SELECT instrument,ipversion from " 
        + obrepDbName + "..obs_descriptions"
        + " WHERE od_id = " + od_id;
        
        ResultSet rs = stmt.executeQuery(sql);
        String instrument = null;
        while(rs.next()) {
           instrument = rs.getString("instrument");
        }
        return instrument;
          
    }
    public static float getIPVFromOd(Statement stmt,long od_id) throws SQLException {
        
        String sql = "SELECT instrument,ipversion from " 
        + obrepDbName + "..obs_descriptions"
        + " WHERE od_id = " + od_id;
        
        ResultSet rs = stmt.executeQuery(sql);
        float ipVersion = 0;
        while(rs.next()) {
           ipVersion = rs.getFloat("ipversion");
        }
        return ipVersion;
          
    }
    
    
    public static float getIPVFromObsRun(Statement stmt,long obsRunId) throws SQLException {
        
       
        String sql = "SELECT ipversion from " 
        + opc70DbName + "..sched_rep"
        + " WHERE run_id = " + obsRunId;
        
        ResultSet rs = stmt.executeQuery(sql);
        float ipVersion = 0;
        while(rs.next()) {
           ipVersion = rs.getFloat("ipversion");
        }
        rs.close();
        return ipVersion;
          
    }    

    
    


    /**
     * retrieves associated periods for an IP
     *
     * @param   instr   instrument name
     * @param   version IP version
     *
     * @return  array of associated periods
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static Vector getPeriods(String instr, float version)
            throws SQLException {

        Connection con = null;
        Statement stmt = null;
        Vector periods = new Vector();

        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()),
                cfg.getDbUserName(), cfg.getDbPassword());
            stmt = con.createStatement();

            String sql = "select period from " + IPP +
                " where inst_name = '" + instr +
                "' and version = " + version;
            stdlog_.debug(sql);
            ResultSet rs = stmt.executeQuery(sql);

            while (rs.next()) {
                Integer i = new Integer(rs.getInt(1));
                periods.addElement(i);
            }

            rs.close();
            stmt.close();
            ConnectionPool.getPool().releaseConnection(con);

        } catch (SQLException e) {
            stdlog_.error("getIP(): " + e.getMessage());
            if (con != null) {
                stmt.close();
                ConnectionPool.getPool().releaseConnection(con);
            }
            throw e;
        }

        return periods;
    }


    /**
     * retrieves IP versions for a given instrument and period
     *
     * @param   instr   instrument name
     * @param   period  associated period
     *
     * @return  array of IP versions
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static Vector getIPversions(String instr, int period)
    throws SQLException {

        Connection con = null;
        Statement stmt = null;
        Vector versions = new Vector();

        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()),
                cfg.getDbUserName(), cfg.getDbPassword());
            stmt = con.createStatement();

            String sql = "select distinct version from " + IPP +
                " where inst_name = '" + instr + "' and period = " + period;
            // if (periodsArray != null && periodsArray.length > 0) {
            //     sql.append(" and period in (" + periodsArray[0]);
            //     for (int i=1; i < periodsArray.length; i++) {
            //         sql.append(", " + periodsArray[i]);
            //     }
            //     sql.append(")");
            // }
            stdlog_.debug(sql);
            ResultSet rs = stmt.executeQuery(sql);

            while (rs.next()) {
                Float f = new Float(rs.getFloat(1));
                versions.addElement(f);
            }

            rs.close();
            stmt.close();
            ConnectionPool.getPool().releaseConnection(con);

        } catch (SQLException e) {
            stdlog_.error("getIP(): " + e.getMessage());
            if (con != null) {
                stmt.close();
                ConnectionPool.getPool().releaseConnection(con);
            }
            throw e;
        }

        return versions;
    }


    /**
     * retrieves all IP versions for a given instrument
     *
     * @param   instr   instrument name
     *
     * @return  vector of IP versions
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static Vector getAllIPversions(String instr) throws SQLException {

        Connection con = null;
        Statement stmt = null;
        Vector versions = new Vector();

        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()),
                cfg.getDbUserName(), cfg.getDbPassword());
            stmt = con.createStatement();

            String sql = "select distinct version from " + IP +
                " where inst_name = '" + instr + "'";
            stdlog_.debug(sql);
            ResultSet rs = stmt.executeQuery(sql);

            while (rs.next()) {
                Float f = new Float(rs.getFloat(1));
                versions.addElement(f);
            }

            rs.close();
            stmt.close();
            ConnectionPool.getPool().releaseConnection(con);

        } catch (SQLException e) {
            stdlog_.error("getIP(): " + e.getMessage());
            if (con != null) {
                stmt.close();
                ConnectionPool.getPool().releaseConnection(con);
            }
            throw e;
        }

        return versions;
    }


    /**
     * used by getIP to convert a UTC date into a string (YYYYMMDD)
     */
    private static String dateString(int addedDate) {
        Calendar c = Calendar.getInstance();
        Date d = new Date((long)addedDate * 1000);
        c.setTime(d);
        int da = c.get(Calendar.DAY_OF_MONTH);
        int mo = c.get(Calendar.MONTH);
        int yr = c.get(Calendar.YEAR);
        mo++;

        String s1 = (mo < 10) ? "0" : "";
        String s2 = (da < 10) ? "0" : "";
        String dateStr = yr + s1 + mo + s2 + da;
        stdlog_.debug("dateStr=" + dateStr);
        return dateStr;
    }


    /**
     * The determination of the "default IP" is performed through a query on
     * the ip_periods tables, filtering for the period and instrument
     * applicable to the observing run.
     *
     * If no version was found for the given instrument and period, -1 is
     * returned.
     *
     * @return the default IP for a given Period and Instrument.
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static float getDefaultIP(String instrument, int period)
            throws SQLException {
        Config cfg = Config.getCfg();
        String dbUrl = cfg.getUrl(ObservationBlock.getSuffix());
        String userName = cfg.getDbUserName();
        String password = cfg.getDbPassword();
        return getDefaultIP(instrument, period, dbUrl, userName, password);
    }


    public static float getDefaultIP(String instrument, int period,
        String dbUrl, String userName, String password) throws SQLException {

        float rc = -1;
        Connection con = null;
        PreparedStatement stmt = null;
        try {
            con = ConnectionPool.getPool().acquireConnection(
                dbUrl, userName, password);

            String sql = "select MAX(version) from " + IPP +
                " where (inst_name = ? and period = ?)";

            stdlog_.debug(sql);
            stmt = con.prepareStatement(sql);

            stmt.setString(1, instrument);
            stmt.setInt(2, period);

            ResultSet rs = stmt.executeQuery();

            // with MAX we expect only a single result row.
            if (rs.next()) {
                rc = rs.getFloat(1);
                // if result wass SQL NULL, 0 is returned, set it to -1
                // to indicate no default IP was found
                if (rc == 0) {
                    rc = -1;
                }
            }
        } finally {
            if (con != null) {
                stmt.close();
                ConnectionPool.getPool().releaseConnection(con);
            }
        }
        return rc;
    }


    /**
     * retrieves a list of Observing Runs from table obs_runs
     *
     * @param instr     instrument name
     * @param version   IP version (can set to unassignedVersion or anyVersion)
     * @param periods   array of periods
     * @return obsRuns list
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static Vector getObsRuns(String instr, float version, int[] periods)
            throws SQLException {

        stdlog_.debug("IPdbio.getObsRuns: " + instr + " v" + version);
        Connection con = null;
        Statement stmt = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()),
                cfg.getDbUserName(), cfg.getDbPassword());
            stmt = con.createStatement();

            StringBuffer sql = new StringBuffer(1000);
            sql.append("select distinct " +
                SR+".id, inst, ipversion, " + SR+".period, " + SR+".progid, " +
                "email_address, " +
                "item_name, surname_dl, email, run_state from " +
                SR + ", " + OR + ", " + OP + ", " + PR + ", " + EU + ", " + RS +
                " where " +
                OR+".id = " + SR+".run_id and " +
                OP+".id = " + OR+".programme_id and " +
                OP+".id = " + PR+".programme_id and " +
                EU+".id = " + PR+".user_id and " +
                OR+".id *=" + RS+".run_id and " +       // outer join
                "pi_flag=0 and " +
                "published=1 and " +
                "inst = '" + instr + "'");
            if (version >= 0) {
                sql.append(" and ipversion = " + version);
            } else if (version == unassignedVersion) {
                sql.append(" and ipversion is Null");
            }
            if (periods != null && periods.length > 0) {
                sql.append(" and " + SR+".period in (" + periods[0]);
                for (int i=1; i < periods.length; i++) {
                    sql.append(", " + periods[i]);
                }
                sql.append(")");
            }
            sql.append(" order by " + SR+".progid, " + SR+".id");
            stdlog_.debug(sql);

            ResultSet rs = stmt.executeQuery(sql.toString());
            Vector obsRunsV = new Vector();
            while (rs.next()) {
                AffectedObsRun obsRun = new AffectedObsRun();
                obsRun.id_ = rs.getInt("id");
                obsRun.period_ = rs.getInt("period");
                obsRun.obsRunName_ = rs.getString("progid");
                obsRun.piInitial_ = rs.getString("item_name");
                obsRun.piName_ = rs.getString("surname_dl");
                obsRun.emailEU_ = rs.getString("email");
                obsRun.emailOP_ = rs.getString("email_address");
                obsRun.runState_ = rs.getString("run_state");
                obsRun.instrument_ = rs.getString("inst");
                obsRun.ipVersion_ = rs.getFloat("ipversion");
                if (rs.wasNull()) {
                    obsRun.ipVersion_ = -1;
                }
                obsRunsV.addElement(obsRun);
            }
            rs.close();
            stmt.close();
            ConnectionPool.getPool().releaseConnection(con);

            return obsRunsV;

        } catch (SQLException e) {
            stdlog_.error("getObsRuns(): " + e.getMessage());
            if (con != null) {
                stmt.close();
                ConnectionPool.getPool().releaseConnection(con);
            }
            throw e;
        }
    }


    /**
     * updates ipversion field for entries in sched_rep table
     *
     * @param IDs               unique identfier (key) in sched_rep table
     * @param ipVersions        new IP version numbers
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static void updateObsRunsIpVersions(int[] IDs, float[] ipVersions)
            throws SQLException {

        Connection con = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()),
                cfg.getDbUserName(), cfg.getDbPassword());
            con.setAutoCommit(false);

            // update one at a time
            for (int i=0; i < IDs.length; i++) {
                String versionStr = ipVersions[i]<0 ? "null" : ipVersions[i]+"";
                String SQLstr = "update " + SR +
                    " set ipversion=" + versionStr +
                    " where id=" + IDs[i];
                stdlog_.debug(SQLstr);
                Statement stmt = con.createStatement();
                stmt.executeUpdate(SQLstr);
                stmt.close();
            }

            // all commands succeeded - commit transaction
            con.commit();
            stdlog_.debug("updateObsRunsIpVersions: success");

        } catch (SQLException e) {
            tryRollback(con, e);
            throw e;

        } finally {
            if (con != null) {
                try  {
                    con.setAutoCommit(true);
                    stdlog_.debug("setAutoCommit: true");
                } catch (SQLException ex) {
                    stdlog_.error("Error setAutoCommit(true):" + ex.getMessage());
                }
                ConnectionPool.getPool().releaseConnection(con);
            }
        }
    }


    /**
     * retrieves either a list of missing IPs or all Obs Runs with missing IPs
     *
     * @param   lite    true: return Missing IPs (instruments and versions only)
     *                  false: return Obs Runs
     * @return missing IPs list
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static Vector getMissingIPs(boolean lite) throws SQLException {
        return getMissingIPs(lite, null);
    }


    /**
     * retrieves either a list of missing IPs or all Obs Runs with missing IPs
     *
     * @param   lite    true: return Missing IPs (instruments and versions only)
     *                  false: return Obs Runs
     * @param   ignoreInstruments   a Vector of String names of instruments that
     *                               are to be ignored (will not appear in the report).
     * @return missing IPs list
     *
     * @exception SQLException  an error occurred during the dbase operation
     */
    public static Vector getMissingIPs(boolean lite, Vector ignoredInstruments)
    throws SQLException {

        stdlog_.debug("IPdbio.getMissingIPs");
        Connection con = null;
        Statement stmt = null;
        boolean haveIgnoredInstruments = false;
        StringBuffer ignoredInstrumentList = new StringBuffer();

        /*
         * See if we have a set of instrument names to be ignored in the report
         * If so, create a list out of them for use in the SQL statement
         */
        if (ignoredInstruments != null) {
            for (Enumeration e = ignoredInstruments.elements(); e.hasMoreElements(); ) {
                String inst = (String)e.nextElement();
                if (inst != null && inst.length() > 0) {
                    if (haveIgnoredInstruments) {
                        ignoredInstrumentList.append(", ");
                    } else {
                        haveIgnoredInstruments = true;
                    }
                    ignoredInstrumentList.append(SqlUtils.sqlEscapedString(inst));
                }
            }
        }

        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()),
                cfg.getDbUserName(), cfg.getDbPassword());
            stmt = con.createStatement();

            String sql = "select distinct inst, ipversion " +
                (lite ? "" : ", progid, period ") +
                "from " + SR + " where published=1 " +
                (haveIgnoredInstruments? "and inst not in ("+ignoredInstrumentList.toString()+") " : "")+
                "and (" +
                    "(ipversion<>null and " + SR+".ipversion not in " +
                        "(select version from " + IP + " where " + IP+".inst_name = " + SR+".inst)" +
                ") or (" +
                "(ipversion=null and period<" + AppConfig.getAppConfig().getIPVersionStartPeriod() +
                    " and 0.0 not in " +
                        "(select version from " + IP + " where " + IP+".inst_name = " + SR+".inst)" +
                "))) order by " +
                (lite ? "inst, ipversion" : "progid, inst, period, ipversion");
            stdlog_.debug(sql);

            ResultSet rs = stmt.executeQuery(sql);
            Vector obsRunsV = new Vector();
            while (rs.next()) {
                AffectedObsRun obsRun = new AffectedObsRun();
                if (!lite) {
                    obsRun.obsRunName_ = rs.getString("progid");
                    obsRun.period_ = rs.getInt("period");
                }
                obsRun.instrument_ = rs.getString("inst");
                obsRun.ipVersion_ = rs.getFloat("ipversion");
                if (rs.wasNull()) {
                    obsRun.ipVersion_ = -1;
                }
                obsRunsV.addElement(obsRun);
            }
            rs.close();
            stmt.close();
            ConnectionPool.getPool().releaseConnection(con);

            return obsRunsV;

        } catch (SQLException e) {
            stdlog_.error("getMissingIPs(): " + e.getMessage());
            if (con != null) {
                stmt.close();
                ConnectionPool.getPool().releaseConnection(con);
            }
            throw e;
        }
    }
}

