package org.eso.ohs.phase2.apps.ot.configreport;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.GTreeNode;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.PAFTable;
import org.eso.ohs.core.utilities.StringSet;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;

/**
 * A ConfigReport generator. This class uses a config report definition
 * and given OBs to generate a String configuration report.
 *
 * Basic Use Case:
 *
 *     // create a ConfigReport.
 *     ConfigReport rep = new ConfigReport();
 *
 *     // add OBs to the report using submitReport()
 *     rep.addOB(obBlk1);
 *     rep.addOB(obBlk2);
 *
 *     // get the text report
 *     String report = rep.generateReport(true);
 *
 * The report definitions are instrument specific and taken from the OHSConfig.
 * Each ConfigReport will generate config reports for the given OBs appropriate
 * the the instrument used in the first OB.
 *
 * ConfigReport therefore does not handle OB sets spanning multiple instruments.
 *
 * The report definition is a set of param names (and sub-names) whose
 * values make up a configuration.
 *
 * The current definition of a report is as follows:
 *     CONFIGREPORT.<instr>.KEY<x>.PARAM<y> "<paramname>"
 *
 * Where:
 *     <instr> is the key for the instrument: FORS2, ISAAC, etc.
 *     <x> is a string, of A-Z, 0-9.
 *     <y> is an index : either blank or 0, 1, 2 ...
 *     <paramname> is the name of a template parameter.
 * 
 * All KEY<idx> values under the instrument will be read as the definition
 * of the configuration report for that instrument.
 * 
 * The KEY suffix (x) can be any value, and is simply an identifier for the key.
 * 
 * The PARAM suffix is used to order the PARAM names under the KEY. This is
 * used to allow multiple parameter names to be used for a config report value.
 * The PARAMs are ordered, and the first non-empty value returned is taken as
 * the report value. For example:
 *     CONFIGREPORT.FORS1.KEY5.PARAM0 "INS.SETUP.VAL1"
 *     CONFIGREPORT.FORS1.KEY5.PARAM1 "INS.ALT.SETUP.VAL1"
 * 
 * If INS.SETUP.VAL1 contains a value for an OB, its value will be used,
 * otherwise the value for INS.ALT.SETUP.VAL1 will be used.
 * 
 * The <paramname> parameter is the name of the parameter to be read from
 * the OBs used to generate the report.
 * It can also contain the name of Paf parameters within a parameter by
 * using the "->" character. For example:
 *     "INS.SETUP1->INS.FILT1"
 * Selects the valus of the INS.FILT1 parameter from within the Paf formetted
 * values of the INS.SETUP1 parameter.
 * 
 * If the sub parameter does not exist, or the main parameter value is not
 * Paf format, an empty value is returned.
 *
 * The following is an example configuration for a config report for
 * ISAAC and FORS2:
 *
 *     CONFIGREPORT.FORS1.KEY1.        "FILT.NAME"
 *     CONFIGREPORT.FORS1.KEY2.        "INS.SETUP.VAL1"
 *     CONFIGREPORT.FORS1.KEY2.PARAM0  "INS.ALT.SETUP.VAL1"
 *     CONFIGREPORT.FORS1.KEY6.PARAM1  "INS.CONFIG->FLT.NAME"
 *     CONFIGREPORT.ISAAC.KEY1         "FILTER.SETTING"
 *     CONFIGREPORT.ISAAC.KEY2         "SETUP.FLT.SLOTS"
 *
 * Extending ConfigReport:
 *
 * The getReportConfig() method should be over-ridden if report
 * configurations are to be obtained from other sources that the OHSConfig,
 * or in other formats in OHSConfig.
 *
 * @author Nick Kornweibel
 */

public class ConfigReport {

    // Vector of ConfigReportParameter objects
    protected Vector config_;
    protected Vector reportSet_ = new Vector();
    protected final String seperator_ = "/";

    /**
     * Standard log channel
     */
    private static Logger stdlog_ = Logger.getLogger( ConfigReport.class );
 
    /**
     * Constructs a ConfigReport.
     */
    public ConfigReport() {
    }

    /**
     * Add a config report for the given OB to the report set.
     */
    public void addOB(CalibrationBlock bo) {
        // Get the correct id number for the report
        long id = bo.getId();
        Media device = Config.getCfg().getMediaFromId(id);
        long idNum;
        if(device.equals(Media.DBASE)){
            idNum = bo.getDbaseId();
        } else {
            idNum = (bo.getDbaseId() != 0)
                    ? Config.getCfg().uniqueToTableId(bo.getDbaseId())
                    : bo.getId();
        if (idNum < 0)
                idNum = 0;
        }
        Vector report = getReport((CalibrationBlock)bo);
        submitOBReport(idNum, bo, report);
    }

    /**
     * Add a config report for the given OB to the report set.
     */
    protected void submitOBReport(long obId, CalibrationBlock blk,
            Vector report) {

        ConfigReportRecord cfgReport = null;
        for (int i = 0; i < reportSet_.size(); i++) {
            ConfigReportRecord rep
                = (ConfigReportRecord)reportSet_.elementAt(i);
            boolean same = true;
            for (int v = 0; v < report.size(); v++) {
                String newVal = (String)report.elementAt(v);
                String repVal = (String)rep.getReportValues().elementAt(v);
                if (!repVal.equals(newVal)) {
                    same = false;
                    break;
                }
            }
            if (same) {
                cfgReport = rep;
            }
        }
        if (cfgReport == null) {
            // new config report.
            cfgReport = new ConfigReportRecord(this);
            cfgReport.values_ = report;
            reportSet_.addElement(cfgReport);
        }
        cfgReport.getOBIds().addElement(""+obId);
        cfgReport.getOBNames().addElement(blk.getName().trim());
        cfgReport.getOBTypes().addElement(blk.getType());
        cfgReport.getOBStatus().addElement(blk.getStatus());
        cfgReport.getOBProgramNames()
                .addElement(blk.getObsRun().getProgId().trim());
        cfgReport.getRAs().addElement(getRA(blk));
    }

    /**
     * @return RA as String, or "" if not found.
     */
    private String getRA(CalibrationBlock blk) {
        String rc;
        Target t = (blk instanceof ObservationBlock)?
            ((ObservationBlock)blk).getTarget() : null;
        if ( t != null) {
            try {
                rc = Convert.milliarcsecToHHMMSS(t.getRA());
            } catch (IllegalArgumentException e) {
                // invalid ra.
                rc = "";
            }
        } else {
            rc = "";
        }
        return rc;
    }

    /**
     * @return Vector of ConfigReportParameter required in the config report,
     * for the given config report definition name.
     *
     * Note that this method initializes the config_ member only once, for
     * the first instrument requested. The ConfigReport supports only a 
     * single instrument.
     *
     * This method should be over-ridden if report configurations are to
     * be obtained from other sources that the OHSConfig, or in other
     * formats in OHSConfig.
     */
    public Vector getReportConfig(String reportName) {
        String inst = reportName;
        if (config_ == null) {
            String path = OHSKeys.ConfigReportKey+"."+inst;
            config_ = new Vector();
            Hashtable params = new Hashtable();
            GTreeNode node = OHSConfig.getInstance().getNode(path);
            stdlog_.debug("Path wanted: "+path);
            stdlog_.debug("Path found: "+node);
            // stdlog_.debug("Config:\n"+OHSConfig.getInstance().toString());
            if (node != null) {
                // we have some report config information.
                for (Enumeration kids = node.getChildren();
                    kids.hasMoreElements();) {
                    GTreeNode child = (GTreeNode)kids.nextElement();
                    if (child.getName().toLowerCase().startsWith("key")) {
                        ConfigReportParameter p = createParameter(child);
                        if (p != null) {
                            params.put(child.getName().toLowerCase(), p);
                        }
                    }
                }
                // copy keys to array and sort the array
                String keys[] = new String[params.size()];
                int i = 0;
                for (Enumeration e = params.keys(); e.hasMoreElements(); i++){
                    keys[i] = (String)e.nextElement();
                }
                keys = TextUtils.sort(keys);
                // place the parameters in the config_ member in the
                // sorted order.
                for (i = 0; i < keys.length; i++) {
                    config_.addElement(params.get(keys[i]));
                }
            }
            stdlog_.debug("\n"+configToString());
        }
        return config_;
    }

    /**
     * @return String report of the config report definition.
     */
    public String configToString() {
        StringBuffer rc = new StringBuffer();
        int i = 1;
        for (Enumeration e = config_.elements(); e.hasMoreElements(); i++ ) {
            ConfigReportParameter p = (ConfigReportParameter)e.nextElement();
            rc.append("Parameter: "+i+" Name: "+p.getName()+"\n");
            for (int x = 0; x < p.getParameterCount(); x++) {
                 rc.append("    TPL:"+p.getTemplateParameter(x)
                     +" PAF:"+p.getPafParameter(x)+"\n");
            }
        }
        return rc.toString();
    }

    /**
     * Read a config tree child and add as a ConfigReportParameter
     * @return a ConfigReportParameter, or null if no valid data found.
     */
    protected ConfigReportParameter createParameter(GTreeNode node) {
     
        ConfigReportParameter rc = null;
        Hashtable params = new Hashtable();
        int pCount = node.getChildCount();
        String nVal = node.getValue();
        if (nVal != null && node.getName().equals(nVal)) {
            // nVal is rubbish if has the same value as the node name
            nVal = null;
        }
        String[] paramNames = null;
        int i = 0;

        if (nVal != null) {
            pCount++;
        }
        paramNames = new String[pCount];
        if (nVal != null && nVal.length() > 0) {
            paramNames[0] = node.getName();
            params.put(node.getName(), nVal);
            i++;
        }

        for (Enumeration e = node.getChildren(); e.hasMoreElements(); ) {
            GTreeNode child = (GTreeNode)e.nextElement();
            String nName = node.getName()+"."+child.getName();
            paramNames[i++] = nName;
            nVal = child.getValue() == null ? "" : child.getValue();
            params.put(nName, nVal);
        }

        // only create a parameter if we have some good data.
        if (params.size() > 0) {
            paramNames = TextUtils.sort(paramNames);
            rc = new ConfigReportParameter((String)params.get(paramNames[0]));
            for (i = 0; i < paramNames.length; i++) {
                rc.addParameter((String)params.get(paramNames[i]));
            }
        }
        return rc;
    }

    /**
     * Add  config definition idx to the provided StringBuffer.
     */
    protected void addConfigDefinition(int idx, StringBuffer out) {
        ConfigReportRecord rep = (ConfigReportRecord)reportSet_.elementAt(idx);
        for (int i = 0; i < config_.size(); i++) {
            ConfigReportParameter param
                = (ConfigReportParameter)config_.elementAt(i);
            String val = (String)rep.getReportValues().elementAt(i);
            StringBuffer line = new StringBuffer();
            line.append(param.getName());
            fillTo(line, 34);
            line.append(" "+val+"\n");
            out.append(line.toString());
        }
        out.append("\n");
    }

    /**
     * add the config report for generated report at index idx to the
     * provided StringBuffer.
     */
    protected void addConfigReport(int idx, StringBuffer out) {
        ConfigReportRecord rep = (ConfigReportRecord)reportSet_.elementAt(idx);
        for (int i = 0; i < rep.getOBIds().size(); i++) {
            StringBuffer line = new StringBuffer();
            line.append((String)rep.getOBIds().elementAt(i));
            String progName = (String)rep.getOBProgramNames().elementAt(i);
            int fillBeforeProgname = 13-progName.length();
            if (fillBeforeProgname < 0) {
                fillBeforeProgname = 0;
            }
            fillTo(line, 10+fillBeforeProgname);
            line.append(" "+progName);
            fillTo(line, 25);
            line.append(" "+(String)rep.getOBNames().elementAt(i));
            String ra = (String)rep.getRAs().elementAt(i);
            int fillBeforeRA = 14-ra.length();
            if (fillBeforeRA < 0) {
                fillBeforeRA = 0;
            } 
            fillTo(line, 57+fillBeforeRA);
            line.append(" "+ra);
            fillTo(line, 76);
            line.append(" "+(idx+1)+"\n");
            out.append(line.toString());
        }
    }

    /**
     * @return Vector - the set of ConfigReportRecord objects used to
     * generate the report.
     */
    public Vector getReportSet() {
        return reportSet_;
    }

    /**
     * Compose the report and present it.
     * @param listObs true if the obs should be listed at the end of the report.
     * @return the completed report.
     */
    public String generateReport(boolean listObs) {
        StringBuffer report = new StringBuffer();
        StringBuffer obList = new StringBuffer();
        report.append("Configuration Report\n\n");
        obList.append("Ob Id      Prog           OB Name"
            +"                                     RA   Config\n");
        obList.append("--------------------------");
        obList.append("--------------------------");
        obList.append("-----------------------------\n");
        for (int i = 0; i < reportSet_.size(); i++) {
            report.append("Configuration                      "+(i+1)+"\n");
            report.append("--------------------------");
            report.append("--------------------------");
            report.append("--------------------------\n");
            addConfigDefinition(i, report);
            report.append("\n");
            if (listObs) {
                addConfigReport(i, obList);
            }
        }
        if (listObs) {
            report.append(obList.toString());
        }
        return report.toString();
    }

    /**
     * Utility method.
     * Insert a String into a Vector, maintaining the order of the
     * Strings in the Vector.
     * The String is not inserted if already an element of the Vector 
     * (it is a Set).
     */
    private void insertIntoSortedSet(Vector v, String s) {
        if (s != null && s.length() > 0) {
            boolean append = true;
            for (int x = 0; x < v.size(); x++) {
                String elem = (String)v.elementAt(x);
                int comp = elem.compareTo(s);
                if (comp == 0) {
                    // already have this value - nothing to do
                    append = false;
                    break;
                } else if (comp > 0) {
                    // new value is less than this item...insert
                    v.insertElementAt(s, x);
                    append = false;
                    break;
                }
            }
            if (append) {
                v.addElement(s);
            }
        }
    }
 
    /**
     * Utility method
     */
    protected static void fillTo(StringBuffer s, int len) {
        int pad = len - s.length();
        while (pad > 0) {
           pad--;
           s.append(" ");
        }
    }

    /**
     * @return true if there exists a report for the name given.
     * This name is taken as the name of the instrument.
     * A valid report definition is a non-null, non-empty Vector.
     */
    public boolean isReportSupported(String name) {
        Vector rDef = getReportConfig(name);
        return (rDef != null && rDef.size() > 0);
    }

    /**
     * @return the Vector of values for the params given in config report
     * definition (obtained in this method with
     * getReportConfig(blk.getOd().getInstrument()).
     */
    public Vector getReport(CalibrationBlock blk) {
        Vector config = getReportConfig(blk.getOd().getInstrument());
        Vector rc = new Vector(config.size());
        // for each param, search through each template and
        // retreive the value for that parameter.
        for (int i = 0; i < config.size(); i++) {
            ConfigReportParameter param = (ConfigReportParameter)config.elementAt(i);
            StringBuffer value = new StringBuffer("");
            Vector values = new Vector();

            // ACQ Template, if appropriate.
            if (blk instanceof ObservationBlock) {
                TemplateSignature tpl =
                        ((ObservationBlock)blk).getTemplateSignature();
                if (tpl != null) {
                    String tVal = getValue(tpl, param);
                    insertIntoSortedSet(values, tVal);
                }
            }

            // OD Templates
            TemplateSignature[] od = blk.getOd().getSignatures();
            for (int t = 0; t < od.length; t++) {
                String tVal = getValue(od[t], param);
                insertIntoSortedSet(values, tVal);
            }

            for (Enumeration e = values.elements(); e.hasMoreElements(); ) {
                String v = (String)e.nextElement();
                if (value.length() > 0) {
                    value.append(seperator_);
                }
                value.append(v);
            }
            if (value.length() == 0) {
                // no value found
                value.append("-");
            }
            rc.addElement(value.toString());
        }
        return rc;
    }

    /**
     * Use ConfigReportParameter p to find a value in this template
     * (possibly reading a PAF Parameter value.
     * @return the value of the param, or "" if not found.
     */
    protected static String getValue(TemplateSignature tpl,
            ConfigReportParameter p) {
        String rc = "";
        // Parameter[] params = tpl.getParamList();
        for (int i = 0; i < p.getParameterCount(); i++) {

            String paramName = p.getTemplateParameter(i);
            String pafParam = null;
            if (p.isPafParameter(i)) {
                pafParam = p.getPafParameter(i);
            }

            String val = tpl.getParameterValue(paramName);

            // Debug output only.
	    
	    /*
	      @author amchavan, 14-Mar-2002: commented out
            stdlog_.debug("Search template "+tpl.getTemplateName()
                +" using report parameter "+p.getName()
                +" (tpl param ["+i+"]: "+paramName+")"
                +((pafParam != null) ? " PAF Param: "+pafParam : "")
                +" tpl value found: "+val);
	    */

            // handle possible Paf values here.
            if (pafParam != null) {
                // should handle it as a PAF format value.
                PAFTable paf = new PAFTable();
                if (!paf.loadFromString(val)) {
                    // not a paf string.
                    stdlog_.debug("Value not recognized as Paramformat.");
                    val = null;
                } else {
                    StringSet s = paf.getValuesForParam(pafParam);
                    if (s.getElementCount() == 0) {
                        stdlog_.debug(
                           "Value recognized as Paramformat, with 0 elements.");
                        val = null;
                    } else {
                        // Paf files can have multiple values for a key,
                        // but we are only interested in the first.
                        val = s.getElementAt(0);
   
                    }
                }
                stdlog_.debug("PAF Param: "+pafParam+" has value: "+val);
            }
            if (val != null && val.length() > 0) {
                rc = val.trim();
                break;
            }
            // break when first match for this template is made
            if (rc.length() > 0) { 
                break;
            }
        }
        return rc;
    }

    /**
     * @return int from String, or -1 if the String is not parsable.
     */
    public static int intFromString(String s) {
        int rc = -1;
        if (s != null) {
            try {
                rc = Integer.parseInt(s);
            } catch (NumberFormatException e) {
                rc = -1;
            }
        }
        return rc;
    }

}
