package org.eso.ohs.phase2.orang;

/**
 *  
 * This class it used to run ranking scripts for ORANG 
 * It is based on ScriptingEVMImpl class but it uses the JaclScriptInterpreter class to execute the scripts
 *
 * @author Paulo Nunes
 */

import java.io.File;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.scripting.JaclScriptInterpreter;
import org.eso.ohs.scripting.ScriptInterpreter;
import org.eso.ohs.scripting.ScriptInterpreterException;
import org.eso.ohs.scripting.ScriptInterpreterManager;

public class OrangScriptingBox implements OBRankingEngine {
   
    /**
     * Config keyword (base) for setup script:
     *     ORANG.SETUP
     */
    public static final String initScriptKeyword = "ORANG.SETUP";


    /** 
     * Script prefix
     */
    public static final String orangScriptPrefix = "orang";
    

    /*
     * Scripts directory
     */
	private String scriptsDir_ ;	
	
    /**
     * Logging instance.
     */
    protected static Logger stdlog_ =
        Logger.getLogger(OrangScriptingBox.class);

    /**
     * Set of OBs that are _currently_ exposed to the scripting env
     */
    private ObScheduleInfoAPI [] obs_ = null;                

    /**
     * The object that locates the scripts 
     */
    private OrangScriptFinder scriptFinder_ = null;
    
    
    private static OrangScriptingBox _instance = new OrangScriptingBox(); 

    
    public static OrangScriptingBox getInstance() { 
    	  return _instance; 
    } 
 

    /**
     * Default constructor - this constructor must (and is implicitly)
     * call the default ScriptingBox constructor in order to set the
     * instance of ScriptingBox available to the scripting environment
     * @throws OrangScriptException 
     */

    private OrangScriptingBox()  {    	    	
    	scriptFinder_ = new OrangScriptFinder();    	  	    
    	setOBsForRanking(null);
    }

    /**
     * Initialize the object with the defined scripting drivers and
     * defined script finder object
     *
     * @param scriptDriver the list of supported ScriptInterpreter
     * classes to be loaded
     * @param script finder object
     * @throws OrangScriptException if initialization fails
     */

    public void initImpl(String[] scriptDrivers, OrangScriptFinder sf)
        throws OrangScriptException {

        if (scriptDrivers == null || scriptDrivers.length == 0 || sf == null) {
            throw new OrangScriptException("Empty or null parameters");
        }
        scriptFinder_ = sf;

        ScriptInterpreterManager man = ScriptInterpreterManager.getInstance();
        try {
            for (int i = 0; i < scriptDrivers.length; i++) {
                if (!man.addScriptInterpreter(scriptDrivers[i])) {
                    throw new OrangScriptException(
                        "Script driver not found: " + scriptDrivers[i]);
                }
            }
        } catch (Exception ex) {
            throw new OrangScriptException(ex.getMessage());
        }
    }

    /**
     * Set OBs to be tested/verified (from start to end, not incl)
     * @param array of OBs
     * @param start index of OBs to be tested
     * @param end index of OBs to be tested
     */

    public void setOBsForRanking(OBScheduleInfo [] obs) {
    	if (obs == null) {
            obs_ = new ObScheduleInfoAPI[0];
        } else {
            obs_ = new ObScheduleInfoAPI[obs.length];
            for (int i = 0; i < obs.length; i++) {
				obs_[i] =  new  OBScheduleInfoAdaptor(obs[i]);
			}
        }
    }

    /**
     * Rank the OBs given. Loop through each OB, finding the scripts
     * to execute. "Expose" the OB to the ScriptingBox environment, and
     * rank the OB.
     */

    public void rank(String algorithmName) throws OrangScriptException {
    	
    	if (obs_ != null) {
    		
    		File script = scriptFinder_.findScript(algorithmName,scriptsDir_);
    		
    		if( script != null) {
    			runScript(script);
            } else {
    			throw new OrangScriptException("No scripts found in " + scriptsDir_);
            }
    	} else {
    		throw new OrangScriptException("OBs to rank not set, please call setOBsForRanking() method");
    	}    
    	
    }


  

    /**
     * Return the set of supported scripting file extensions
     * @return set of supported script file extensions
     */

    public static String[] getSupportedScriptExtensionsImpl() {
        return ScriptInterpreterManager.getInstance().getFileExtensions();
    }

    /**
     * Run a script
     * @param the script File to be run
     * @throws if an error occurs executing the script
     */

    public void runScript(File script) throws OrangScriptException {
        ScriptInterpreterManager man = ScriptInterpreterManager.getInstance();
        // extract the file extension
        String fileName = script.getName();
        stdlog_.debug("Filename " + fileName);
        int extDot = fileName.lastIndexOf('.');
        String ext = null;
        if (extDot < 0) {
            // force an index oob exception below
            extDot = fileName.length();
        }
        try {
            ext = fileName.substring(extDot + 1);
        } catch (StringIndexOutOfBoundsException ex) {
            throw new OrangScriptException(
                "No script driver found for: " + script.getPath());
        }
        ScriptInterpreter si = man.selectInterpreterByFileExt(ext);
        if (si == null) {
            throw new OrangScriptException(
                "No script driver found for: " + script.getPath());
        } else {
            try {
                stdlog_.debug("Executing script");
                si.executeFile(script);
            } catch (ScriptInterpreterException siex) {
                throw new OrangScriptException(siex.getMessage());
            }
        }
    }

    

    /**
     * Required by ScriptingBox
     * Get the number of Observation Blocks exposed to the scripting
     * environment. This method must be implemented.
     * @return array of ObservationBlockAPI objects
     */

    public int getNumberOfObservationBlocks() {
        return obs_.length;
    }

    /**
     * Required by ScriptingBox
     * Get the ObservationBlock at the specified index (>=0)
     * This method must be implemented.
     * @param the index of the OB
     * @return the ObservationBlockAPI at that index, or null
     */

    public ObScheduleInfoAPI getObSchedInfoAt(int i) {
    	return(obs_[i]);
    }
    

	public void setup(String scriptsDir) throws OrangScriptException {
        
		scriptsDir_ = scriptsDir;
		
    	OHSConfig config = OHSConfig.getInstance();
    	    	
    	ScriptInterpreterManager man = ScriptInterpreterManager.getInstance();    	
    	man.addScriptInterpreter(JaclScriptInterpreter.class.getName());
    	    	     	    
    	String  initScript = getInitScript(config);    	    	    	    	
    	runScript(new File(initScript));
    	    	
    	
    }

    /**
     * Extract the init script info from the config info
     * @param the config data
     * @return script file name
     * @throws OrangScriptException 
     */

    private  String getInitScript(OHSConfig config) throws OrangScriptException {
          String script = config.getValue(initScriptKeyword);    
        
          if ( script != null) {
        	  if (script.length() > 0) 
        		  return(script);
          }
          else
        	  throw new OrangScriptException("ORANG: Couldn't find init script");
        			  
        return script;
    }


    
    
    public void setOBScore(int index,String score) {
    	obs_[index].setRankScore(score);
    	stdlog_.debug("\nSet OB #"+index+" score:" + score);
    }
    
    public String getOBScore(int index) {
    	stdlog_.debug("Return OB #"+index+" score:" + obs_[index].getRankScore());
    	return(obs_[index].getRankScore());
    }



    public OBScheduleInfo[] getRankedOBs() {
    	
    	OBScheduleInfo [] res = new OBScheduleInfo[obs_.length];
    	
    	for (int i = 0; i < obs_.length; i++) {
    		res[i] = obs_[i].getObSchedInfo();
		}
    	
    	return(res);
        
    }



	public String[] getAlgorithmNames() throws OrangScriptException {
		String[] scripts = new String[0];
  
        scripts = scriptFinder_.findNames(scriptsDir_);
   
		return(scripts);
		
	}
    
    
    
    

}
