package org.eso.ohs.phase2.visibility;

/**
 * PCF - class to store a piece-wise continuous function.
 * PCFs are Vector of start-time and floating point value pairs.
 * e.g. '(1 2.0 9 10.1) is a PCF with values:
 *  from times: 1 to 9; value 2.0
 *  from time: 9 to PLUS-INFINITY; value 10.1
 *
 * Times are stored as Date objects, and values are stored as floating point
 * numbers.
 *
 * Constraints:
 * - time values must be non-decreasing
 * - there must be at least one time-value pair
 * -"spikes" are presently NOT ALLOWED, e.g.
 * '(0 0.0 5 1.0 5 0.0) represents a PCF with a value of 1.0 at exactly 5,
 * but 0.0 everywhere else.  Spikes are indicated by t(i) = t(i+1)
 *
 * Date values are stored as the long millisecond values (since UNIX epoch).
 * The Date value 0L is used for negative infinity time 
 * (i.e. the lowest possible time boundary).
 * The Date value Long.MAX_VALUE is used for positive infinity time 
 * (i.e. the highest possible time boundary).
 *
 * Note that the PCFPoint is used for internal storage of values, not
 * the PCFInterval. PCFInterval is used only for external representation
 * of an interval.
 *
 * @author N. Kornweibel for ESO.
 * <br>ChangeLog: cmavros 27/09/01 Add define value to exclude, confine, and filter
 */

import java.text.ParseException;
import java.util.Date;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.Convert;


public class PCF {


   /**
    * Standard log channel
    * <br>ChangeLog: cmavros 21-12-01 added
    */
   private static Logger stdlog_ =
   Logger.getLogger( PCF.class );

    /**
     * Vector for storing the PCF.
     */
    protected Vector pcf_ = new Vector();

    /**
     * Maximum upper bound of time values.
     */
    public static final Date PositiveInfinity = new Date(Long.MAX_VALUE);

    /**
     * Minimum lower bound of time values.
     */
    public static final Date NegativeInfinity = new Date(0L);

    /**
     * Default value of an interval.
     */
    public static final double DefaultValue = 0.0;

    /**
     * Create a PCF with a constant value.
     * @param initial default value for PCF.
     */

    public PCF(double value) {
        pcf_.addElement(new PCFPoint(NegativeInfinity, value));
    }

    /**
     * Create a PCF with a constant value, the default value.
     */

    public PCF() {
        pcf_.addElement(new PCFPoint(NegativeInfinity, DefaultValue));
    }

    /** 
     * Create a PCF by parsing a String.
     * 
     *  @param in   A String like <code>(2.0 9 10.1)</code>; it is assumed
     *              that it represents a PCF extending from negative
     *              infinity to positive infinity.
     *              <br>
     *              There should be an odd number of elements in the String,
     *              separated by (any amount of) whitespace.
     *              <br>
     *              Elements of the input String in even positions
     *              denote interval values and can be any real number; 
     *              those in odd positions denote interval boundaries and 
     *              must be integers in increasing order, or ISO dates in
     *              increasing order.
     *              <br>
     *              For instance:
     *              <ul>
     *              <li> <code>(1)</code>: a PCF extending from 
     *                   negative to positive infinity with value 1.0s
     *                   
     *              <li> <code>(3.0 1 2.0)</code>: a PCF extending from 
     *                   negative to 1 with value 3.0, and from time 1 to
     *                   positive infinity with value 2.0
     *                   
     *              <li> <code>(46 2005-10-16T02:00:00 47)</code>: a 
     *                   PCF extending from 
     *                   negative to Maurizio's birthday with value 46, 
     *                   and from that time to
     *                   positive infinity with value 47
     *              </ul>
     *              
     * @throws IllegalArgumentException If syntax of input string is invalid
     * @throws NumberFormatException    If input string contains anything else 
     *                                  than reals and integers or ISO dates
     *                                  in the proper positions
     * 
     */
    // amchavan, 31-Aug-2005: added
    public PCF( String in ) 
        throws IllegalArgumentException, NumberFormatException {
        
        final String SYNTAX = "Invalid syntax";
        
        if( in == null || in.length() == 0 )
            throw new IllegalArgumentException( "Null or empty input" );

        in = in.trim();
        if( in.charAt( 0 ) == '(' )
            in = in.substring( 1 ).trim();
        else
            throw new IllegalArgumentException( SYNTAX );
        
        int index = in.length() - 1;
        if( in.charAt( index ) == ')' )
            in = in.substring( 0, index ).trim();
        else
            throw new IllegalArgumentException( SYNTAX );
        
//        System.out.println( in );
        
        // Break down input string in tokens: they must be an odd number,
        // like (1), or (3.0 1 2.0)
        String[] elements = in.split( "\\s" );
        if( elements.length % 2 == 0 )
            throw new IllegalArgumentException( SYNTAX );
        
        // Add initial element to the PCF
        long curtime = 0L;
        add( curtime, elements[0] );
        
        // Add all other element pairs to the PCF
        int i = 1;
        while( i < elements.length ) {
            String timestring = elements[i];
            long time = 0;
            if( timestring.indexOf( "T") != -1 )    // is this an ISO date?
                try {
                    // yes, try to convert that to an integer
                    time = Convert.ISODateToMilliSec( timestring );
                }
                catch( ParseException e ) {
                    throw new IllegalArgumentException( SYNTAX + ":" + 
                                                        e.getMessage() );
                }
            else {
                // no, assume it *is* an integer
                time = Long.valueOf( elements[i] ).longValue();
            }
            if( time > curtime )
                curtime = time;
            else
                throw new IllegalArgumentException( "Non-increasing times" );
            add( time, elements[i+1] );
            i += 2;
        }
    }
    
    /** Add an element to this PCF from two strings */
    private void add( long time, String value ) 
        throws NumberFormatException {
        double v = Double.valueOf( value ).doubleValue();
        pcf_.addElement(new PCFPoint( new Date( time ), v ));        
    }
    
    /**
     * Insert an interval into the PCF.
     *
     * Invalid insertion:
     * The value being added spans more than 1
     * interval in the existing PCF, and therefore can not be added:
     *
     * Existing PCF:
     *        +-------+
     *   -----+       +----
     * Value to be added:
     *      +---+
     *      |   |
     * Result: addition not possible.
     *
     * Valid addition of value into PCF:
     *
     * Existing PCF:
     *        +-------+
     *   -----+       +----
     * Value to be added:
     *          +---+
     *          |   |
     * Result:
     *          +---+
     *        +-+   +-+
     *   -----+       +----
     *
     * @param start time of the interval
     * @param end time of the interval
     * @param value to be inserted.
     * @return true if sucessfully inserted, else false.
     */

    public boolean addInterval(Date startTime, Date endTime, double value) {
        //stdlog_.info("add start: "+startTime.getTime()
        //        +" end: "+endTime.getTime()+" val: "+value);
        boolean rc = true;
        if (endTime.before(startTime) || endTime.equals(startTime)) {
            // start time is after or equal to end time.
            rc = false;
        } else {
            int startIndex = findInsertionIndex(startTime, false);
            int idx = findInsertionIndex(endTime, true);
            if (startIndex != idx) {
                /*
                 * Invalid insertion:
                 * The value being added spans more than 1
                 * interval, and therefore can not be added.
                 */
                rc = false;
            } else {
                /*
                 * Valid insertion.
                 */
               
                /* 
                 * Add the end point, if required.
                 */
                if (!dateIsPositiveInfinity(endTime)
                        && (idx == pcf_.size()
                            || endTime.before(
                            ((PCFPoint)pcf_.elementAt(idx)).getStart()))) {
 
                    PCFPoint endPt = null;
                    if (idx == 0) {
                        endPt = new PCFPoint(endTime,
                            ((PCFPoint)pcf_.elementAt(idx)).getValue());
                    } else {
                        endPt = new PCFPoint(endTime,
                            ((PCFPoint)pcf_.elementAt(idx-1)).getValue());
                    }
                    pcf_.insertElementAt(endPt, idx);
                }

                /*
                 * Modify an existing point, or insert a new one as required.
                 */
                int indexBefore = idx;
                if (idx != 0) {
                    indexBefore--;
                }
                PCFPoint startPoint = (PCFPoint)pcf_.elementAt(indexBefore);
                if (startTime.equals(startPoint.getStart())) {
                    startPoint.setValue(value);
                } else {
                    startPoint = new PCFPoint(startTime, value);
                    pcf_.insertElementAt(new PCFPoint(startTime, value), idx);
                }

                rc = true;
            }
        }
        return rc;
    }

    /**
     * Add an interval to the PCF. See addInterval() above.
     * @param interval object to be added.
     * @return true if added suceeded, else false.
     */
    
    public boolean addInterval(PCFInterval p) {
        return addInterval(p.getStart(), p.getEnd(), p.getValue());
    }

    /**
     * Add an interval which extends to +ve infinity.
     * @param start time of the interval
     * @param value of the interval
     * @return true if sucessfully added, else false.
     */

    public boolean addInterval(Date startTime, double value) {
        return addInterval(startTime, PositiveInfinity, value);
    }

    /**
     * Append an interval to the PCF - this is added _before_ the interval
     * which extends to PositiveInfinity.
     * That is, appending: (10, 3.0) 
     * To: (2 1.0 5 2.0)
     * Gives: (2 1.0 5 3.0 10 2.0).
     *
     * @return true of the append was sucessful, else false.
     * @param the end time of the interval to be added.
     * @param the value of the appended interval.
     */

    public boolean appendInterval(Date endTime, double value) {
        PCFPoint p = (PCFPoint)pcf_.elementAt(pcf_.size()-1);
        return addInterval(p.getStart(), endTime, value);
    }

    /**
     * Return the index position in the pcf where the specified Date 
     * would be inserted (to maintain non-decreasing time-line).
     *
     * @param date the date value to be used as the insertion point
     * @param firstPossible true if the last possible insertion 
     * point should be found, instead of the first possible insertion point.
     * @return the value where the date would be inserted
     */

    public int findInsertionIndex(Date date, boolean firstPossible) {
        Enumeration e = pcf_.elements();
        int rc = 0;
        do {
            PCFPoint p = (PCFPoint)e.nextElement();
            if (date.before(p.getStart())) {
                break;
            }
            if (firstPossible && date.equals(p.getStart())) {
                break;
            }
            rc++;
        } while (e.hasMoreElements());
            //stdlog_.info("@@ insert index for date: "+date.getTime()
            //+": "+rc);
        return rc;
    }

    /**
     * Remove intervals that are redundant.
     * A redundant interval is an interval with the same value as
     * the interval before it.
     */

    public void pack() {
        double lastValue = 0.0;
        int idx = 0;
        
        if (pcf_.size() > 0) {
        	do {
        		if (idx == 0) {
        			lastValue = ((PCFPoint)pcf_.elementAt(idx)).getValue();
        			idx++;
        		} else {
        			double val = ((PCFPoint)pcf_.elementAt(idx)).getValue();
        			if (lastValue == val) {
        				pcf_.removeElementAt(idx);
        			} else {
        				lastValue = val;
        				idx++;
        			}
        		}
        	} while (idx < pcf_.size()); 
        }
    }
     

    /**
     * @return the value for the PCF given the time.
     */

    public double getValueAt(Date time) {
        int index = findInsertionIndex(time, false);
        if (index != 0) {
            index--;
        }
        return ((PCFPoint)pcf_.elementAt(index)).getValue();
    }

    /**
     * @return the number of intervals in this PCF.
     */

    public int getIntervalCount() {
        return pcf_.size();
    }

    /**
     * @return the PCFPoint object for the specified interval, i
     * @param the index of the interval to be returned.
     */
    public PCFPoint getPointAt(int i) {
        return (PCFPoint)pcf_.elementAt(i);
    }

    /** @return The earliest interval having the highest possible value. */
    public PCFInterval getEarliestMax() {
        return getMax( 0, getIntervalCount() - 1, +1 );
    }
    
    /** @return The earliest interval having the lowest possible value. */
    public PCFInterval getEarliestMin() {
        return getMin( 0, getIntervalCount() - 1, +1 );
    }
    
    /** 
     * @return The earliest interval having the lowest possible value.
     * @param duration  Only intervals longer than this many milliseconds
     *                  will be considered
     */
    public PCFInterval getEarliestMin( long duration ) {
        return getMin( 0, getIntervalCount() - 1, +1, duration );
    }
    
    /** @return The latest interval having the highest possible value. */
    public PCFInterval getLatestMax() {
        return getMax( getIntervalCount() - 1, 0, -1 );
    }
    
    /** @return The latest interval having the lowest possible value. */
    public PCFInterval getLatestMin() {
        return getMin( getIntervalCount() - 1, 0, -1 );
    }
    
    private PCFInterval getMax( int start, int end, int increment ) {
        PCFInterval ret = null;
        double max = Double.NEGATIVE_INFINITY;
        
        int i = start;
        while( true ) {
            
            // Termination condition depends on which direction we go
            if( increment > 0 && i > end )
                break;
            if( increment < 0 && i < end )
                break;
            
            PCFInterval current = getIntervalAt( i );
            
            if( ret == null ) {
                ret = current;
                max = ret.getValue();
                continue;
            }
            
            if( current.getValue() > max ) {
                ret = current;
                max = current.getValue();
            }
            
            i += increment;
        }
        
        return ret;
    }

    private PCFInterval getMin( int start, int end, int increment ) {
        return getMin( start, end, increment, 0L );
    }
        
    private PCFInterval getMin( int start, int end, int increment, long duration ) {
        PCFInterval ret = null;
        double min = - Double.NEGATIVE_INFINITY;
        
        int i = start;
        while( true ) {
            
            PCFInterval current = getIntervalAt( i );
            i += increment;
            long currDuration = current.getEnd().getTime() - 
                                current.getStart().getTime();
            double currValue = current.getValue();
            
            if( (ret == null || currValue < min) && currDuration >= duration ) {
                ret = current;
                min = ret.getValue();
            }
                        
            // Termination condition depends on which direction we go
            if( increment > 0 && i > end )
                break;
            if( increment < 0 && i < end )
                break;
            
        }
        
        return ret;
    }

    
    /**
     * @return the PCFInterval object for the specified interval, i
     * @param the index of the interval to be returned.
     */

    public PCFInterval getIntervalAt(int i) {
        PCFPoint p = (PCFPoint)pcf_.elementAt(i);
        Date startDate = p.getStart();
        double value = p.getValue();
        Date endDate = null;
        i++;
        if (i < pcf_.size()) {
            endDate = ((PCFPoint)pcf_.elementAt(i)).getStart();
        } else {
            endDate = PositiveInfinity;
        }
        return new PCFInterval(startDate, endDate, value);
    }

    /**
     * Set a value in the PCF - over writing what is currently there
     * @param interval object to be set.
     * @return true if set suceeded, else false.
     */
    
    public boolean setInterval(PCFInterval p) {
        return setInterval(p.getStart(), p.getEnd(), p.getValue());
    }    
    
    /**
     * Set a value in the PCF - over writing what is currently there
     * @param startTime
     * @param endTime
     * @param value
     * @return true if set succeeded, else false.
     */

    public boolean setInterval( Date startTime, Date endTime, double value ) {

        if( endTime.before( startTime ))
            return false;
        
        int startIndex = findInsertionIndex( startTime, true );
        int endIndex = findInsertionIndex( endTime, false );

        // Special case: if the interval spans this entire PCF
        // we can (must, actually) take a shortcut
        // amchavan, 09-Nov-2005
        //----------------------------------------------------
        if( startIndex == 0 && endIndex == getIntervalCount() ) {
            pcf_.clear();
            pcf_.addElement( new PCFPoint( startTime, value ));
            return true;
        }
        
        if( startIndex != endIndex ) {
            // If the interval does /not/ extend to positive infinity
            // we need to mangle the end point; otherwise we better
            // leave it alone.
            // amchavan, 02-Nov-2005
            // -------------------------------------------------------
            if( !endTime.equals( PositiveInfinity )) {
                // reset the end time of the last point in the interval
                // covered by the interval.
                endIndex--;
                PCFPoint p = (PCFPoint) pcf_.elementAt( endIndex );
                p.setStart( endTime );
            }
            // remove all other points covered in the set.
            while( endIndex > startIndex ) {
                pcf_.removeElementAt( startIndex );
                endIndex--;
            }
        }
        
        boolean rc = addInterval( startTime, endTime, value );
        return rc;
    }

    /**
     * Normalize around an absolute span value of 1.0.
     */

    public void normalize() {
        normalize(1.0, 0.0);
    }

    /**
     * Normalize the PCF use absSpan as the maximum absolute span value
     * and centering around zero.
     * If absValue is 0.0, nothing is performed.
     *
     * @param absSpan the absolute maximum value.
     */

    public void normalize(double absSpan) {
        normalize(absSpan, 0.0);
    }

    /**
     * Normalize the PCF use absSpan as the maximum absolute span value
     * and centering around zero.
     * If absValue is 0.0, nothing is performed.
     *
     * @param absSpan the absolute maximum value.
     * @param the zero value.
     */

    public void normalize(double absSpan, double zero) {
        // avoid division by 0.
        if (absSpan != 0.0) {
            for (Enumeration e = pcf_.elements(); e.hasMoreElements(); ) {
                PCFPoint p = (PCFPoint)e.nextElement();
                p.setValue((p.getValue()-zero)/absSpan);
            }
        }
    }

    /**
     * @return a string form of this PCF.
     */
    public String toStringOLD() {
        StringBuffer rc = new StringBuffer();
        rc.append("Time\t\tValue\n");
        for (Enumeration e = pcf_.elements(); e.hasMoreElements();) {
            PCFPoint p = (PCFPoint)e.nextElement();
            if (dateIsPositiveInfinity(p.getStart())) {
                rc.append("+ve inf");
            } else if (dateIsNegativeInfinity(p.getStart())) {
                rc.append("-ve inf");
            } else {
                rc.append(""+p.getStart());
            }
            rc.append("\t\t"+p.getValue()+"\n");
        }
        return rc.toString();
    }

    /**
     * @return A string representation of the input PCF of
     *         the form <em>(date1 value1 date2 value2)</em>
     */
    public String toString() {
        StringBuffer rc = new StringBuffer();
        rc.append( "(" );
        int n = this.getIntervalCount();
        PCFPoint p;
        for( int i = 0; i < n; i++ ) {
            p = this.getPointAt( i );

            Date d = p.getStart();
            // Try to present something understandable...
            // amchavan, 19-Oct-2005
            long t = d.getTime();
            String rep;
            if( t < 1000000 )
                rep = String.valueOf( (int) t );
            else
                rep = Convert.longToISODateMilliSec( t );
            
            if( i > 0 ) {
                rc.append( " " );
                rc.append( rep )
                  .append( " " )
                  .append( p.getValue() );
            }
            else 
                rc.append( p.getValue() );
        }
        rc.append( ")" );
        return rc.toString();
    }
    
    public String toStringCommas() {
    	StringBuffer rc = new StringBuffer();        
        int n = this.getIntervalCount();
        PCFPoint p;
        for( int i = 0; i < n; i++ ) {
            p = this.getPointAt( i );                    
                rc.append( p.getStart() )
                  .append( "," )
                  .append( p.getValue() )
                  .append( "\n" );
        }
        
        rc.append("\n");
        return rc.toString();
    }

    /**
     * @return true if the provided date is equal to the definition
     * of Positive Infinity.
     */

    //TODO
    // Hack! This problem should be solved in another way!
    // pnunes 12.10.2005
    public static boolean dateIsPositiveInfinity(Date d) {
    	return( Math.abs(d.getTime() - PositiveInfinity.getTime()) < (1000*60*60*24*2) );    		    	
        //return d.equals(PositiveInfinity);
    }

    /**
     * @return true if the provided date is equal to the definition
     * of Negative Infinity.
     */

    public static boolean dateIsNegativeInfinity(Date d) {
        return d.equals(NegativeInfinity);
    }

    /**
     * Reduce all values in this PCF to 0.0 for periods where
     * the excluder PCF has non-0.0 values.
     *
     * @param confiner PCF defining intervals to which this PCF
     * is to exclude.
     */

    public void exclude(PCF excluder) {
        filter(excluder, true);
    }

    /**
     * Reduce all values in this PCF to 0.0 for periods where
     * the excluder PCF has non-0.0 values.
     *
     * @param excluder PCF defining intervals to which this PCF
     * @param value a defined value 
     */

    public void exclude(PCF excluder, double value) {
        filter(excluder, true, value);
    }

    /**
     * Reduce all values in this PCF to 0.0 except for periods where
     * the confiner PCF has non-0.0 values.
     *
     * @param confiner PCF defining intervals to which
     * this PCF is to be confined.
     */

    public void confine(PCF confiner) {
        filter(confiner, false);
    }

    /**
     * Reduce all values in this PCF to the input value except for periods where
     * the confiner PCF has non-0.0 values.
     * 
     * @param confiner PCF defining intervals to which this PCF is to be confined.
     * @param value    A value
     */

    public void confine(PCF confiner, double value) {
        filter(confiner, false, value);
    }

    /**
     * Perform confine or exclude operations on this PCF according
     * to the values in the filterDef PCF and the exclude parameter.
     *
     * @param filterDef PCF defining the intervals to be included/excluded.
     * @param exclude operation to be performed (confine or exclude).
     */
    // amchavan, 18-Oct-2005: now calling filter( PCF, boolean, double )
    private void filter( PCF filterDef, boolean exclude ) {
        filter( filterDef, exclude, 0.0 );
    }

    /**
     * Perform confine or exclude operations on this PCF according
     * to the values in the filterDef PCF and the exclude parameter.
     *
     * @param filterDef PCF defining the intervals to be included/excluded.
     * @param exclude operation to be performed (confine or exclude).
     * @param setValue a defined value 
     */

    private void filter(PCF filterDef, boolean exclude, double setValue) {
        boolean confine = !exclude;     // for readability
        for (int i = 0; i < filterDef.getIntervalCount(); i++) {
            PCFInterval p = filterDef.getIntervalAt(i);
            if ((exclude && p.getValue() != 0.0)
                    || (confine && p.getValue() == 0.0)) {
                setInterval(p.getStart(), p.getEnd(), setValue);
            }
        }
    }
    
    /**
     * Modify this PCF as the union of this and the other PCF.
     * If and interval in the other PCF is > 0.0, the
     * interval is set in this PCF, with the value specified.
     * @param other
     * @param setValue
     */
    
    public void add(PCF other, double setValue) {
        for (int i = 0; i < other.getIntervalCount(); i++) {
            PCFInterval p = other.getIntervalAt(i);
            if (p.getValue() != 0.0) {
                setInterval(p.getStart(), p.getEnd(), setValue);
            }
        }
    }

    /**
     * Set all intervals with values not more than the constraint value to 0.0.
     */

    public void filter(double threshold) {
        for (int i = 0; i < getIntervalCount(); i++) { 
            PCFInterval p = getIntervalAt(i);
            if (p.getValue() <= threshold) {
                setInterval(p.getStart(), p.getEnd(), 0.0);
            }
        } 
    }

    /**
     * @return a deep copy of this PCF, overrides Object.clone().
     */

    public Object clone() {
        PCF rc = new PCF();
        for (int i = 0; i < getIntervalCount(); i++) {
            // amchavan, 30-Aug-2005: was not cloning deep enough
            PCFInterval p = (PCFInterval) getIntervalAt(i).clone();
            rc.addInterval(p);
        }
        return rc;
    }
      
    /**
     * Builds a binary PCF using an array as initializer
     * and a constraint value to decide wether to store 1 or 0 
     * 
     * @param begin
     * @param end
     * @param minutesInterval
     * @param values
     * @param constraint
     * @param greaterThan
     */
    public void buildFromArrayConstraint(Date begin, int minutesInterval ,double [] values, double constraint, boolean greaterThan ) {
    	int msecsInterval = VisibilityCalcUtils.MILLISECS_PER_SEC * 60 * minutesInterval;
    	
    	pcf_ = new Vector();
    	
    	Date d = begin;
    	
    	for (int i = 0; i < values.length; i++) {
    		double value = values[i];
    		PCFPoint point;
    		
    		if ( greaterThan ) {
    			if ( value > constraint )
    				point = new PCFPoint(d,1.0);
    			else
    				point = new PCFPoint(d,0.0);
    			
    		} else {
    			if ( value < constraint )
    				point = new PCFPoint(d,1.0);
    			else
    				point = new PCFPoint(d,0.0);
    		}
    		
    		pcf_.add(point);
    		d = new Date(d.getTime() + msecsInterval  );
    	}
    }
    
    
    /**
     * Builds a binary PCF using an array as initializer
     *
     * @param begin
     * @param end
     * @param minutesInterval
     * @param values
     */
    public void buildFromArray(Date begin,  int minutesInterval ,double [] values) {
    	int msecsInterval = VisibilityCalcUtils.MILLISECS_PER_SEC * 60 * minutesInterval;
    	
    	pcf_ = new Vector();
    	
    	Date d = begin;
    	for (int i = 0; i < values.length; i++) {
    		double value = values[i];
    		
  			PCFPoint point = new PCFPoint(d,value);
    		pcf_.add(point);	
    		d = new Date(d.getTime() + msecsInterval  );
    	}
    	
    }
    
    private double findValueAt(Date startDate) {
    	
    	for (Iterator iter = pcf_.iterator(); iter.hasNext();) {
			PCFPoint pcfPoint = (PCFPoint) iter.next();

			long msecsDate = startDate.getTime();
			long msecsPoint = pcfPoint.getStart().getTime();
			if ( msecsDate == msecsPoint ) {
				return pcfPoint.getValue();
			}
		}

        return -1;
    }

    /**
     * @return <code>true</code> if the other PCF is, interval by interval,
     *         the same as this one; <code>false</code> otherwise.
     */
    // Introduced because equals() does not do the right thing, in my opinion.
    // amchavan, 20-Oct-2005
    public boolean equals2( PCF other ) {
        final double TOLERANCE = 10E-4;
        int count = getIntervalCount();
        if( other.getIntervalCount() != count )
            return false;

        for( int i = 0; i < count; i++ ) {
            PCFPoint myPoint = getPointAt( i );
            PCFPoint otherPoint = other.getPointAt( i );

            if( myPoint.getStart().getTime() != otherPoint.getStart().getTime() )
                return false;
            
            double myValue = myPoint.getValue();
            double otherValue = otherPoint.getValue();
            if( Math.abs( myValue - otherValue ) > TOLERANCE )
                return false;
        }
        return true;
    }
    
    public boolean equals(Object pcf) {
        PCF pcfObj = null;
        boolean result = true; 
        if (pcf instanceof PCF ) {
        	pcfObj = (PCF) pcf;
        	
        	for (int i = 0; i < getIntervalCount();i++){
        		
        		PCFPoint mypoint = getPointAt(i);
        		Date mydate = mypoint.getStart();
        		double myvalue = mypoint.getValue();                 
        		double othervalue = pcfObj.findValueAt(mydate);
        		
        		if ( othervalue == -1 ) {
        			result = false;
        			break;
        		}
        			
        		if ( Math.abs(othervalue - myvalue) > 0.0001 ){
        			result = false;
        			break;
        		}
        	}
        	
        	
        } else {
        	result = false;
        }
        return result;
    }      
    
    public PCFPoint [] getPoints() {
    	int count = this.pcf_.size();
    	PCFPoint [] points = new PCFPoint[count]; 
    	
    	
    	for (int i = 0; i < count; i++) {
			PCFPoint p  = this.getPointAt(i);
			points[i] = p;
		}
    	
    	return(points);
    }
    
    
}
