package org.eso.ohs.phase2.visibility;

/**
 * An interval in a PCF. Contains a start time, end time and value.
 * If the start time and the end time are the same, the interval is
 * termed a "spike" - see PCF.java for more details on PCFs.
 *
 * @see PCF
 * @author N. Kornweibel for ESO.
 */

import java.util.Date;

import org.eso.ohs.core.utilities.Convert;

public class PCFInterval {

    private Date startDate_;
    private Date endDate_;
    private double value_;

    /**
     * Create a PCFInterval with the specified start, end dates and value.
     */

    public PCFInterval(Date startDate, Date endDate, double value) {
        init(startDate, endDate, value);
    }

    /**
     * Create a PCFInterval with the specified start, date and value.
     * The end date will be initialized to PCF.PositiveInfinity.
     */

    public PCFInterval(Date startDate, double value) {
        init(startDate, PCF.PositiveInfinity, value);
    }

    /**
     * Private initializer for data members.
     */

    private void init(Date startDate, Date endDate, double value) {
        startDate_ = startDate;
        endDate_ = endDate;
        value_ = value;
    }

    
    /**
     * @return the start date of the interval.
     */

    public Date getStart() {
        return startDate_;
    }

    /**
     * @return the end date of the interval.
     */

    public Date getEnd() {
        return endDate_;
    }

    /**
     * @return the value of the interval.
     */

    public double getValue() {
        return value_;
    }
    
    /** @return <code>true</code> if the input date is included
     *          in this interval.
     */
    // amchavan, 30-Aug-2005: added
    public boolean includes( Date d ) {
        return (d.compareTo( startDate_) >= 0) && (d.compareTo( endDate_) <= 0);
    }
    
    /** Overrides Object.clone() */
    // amchavan, 30-Aug-2005: added
    public Object clone() {
        PCFInterval ret = new PCFInterval( getStart(), getEnd(), getValue() );
        return ret;
    }
    
    /** Overrides Object.toString() */
    // amchavan, 30-Aug-2005: added
    public String toString() {
        long from = getStart().getTime();
        String sFrom = from < 1000000 ? String.valueOf( from )
                                      : Convert.longToISODateMilliSec( from );

        long to= getEnd().getTime();
        String sTo = to < 1000000 ? String.valueOf( to )
                                  : Convert.longToISODateMilliSec( to );
        
        StringBuffer sb = new StringBuffer();
        sb.append( "[" )
          .append( "from=" ).append( sFrom )
          .append( ",to=" ).append( sTo )
          .append( ",value=" ).append( getValue() )
          .append( "]" );
        return sb.toString();
    }
}
