package org.eso.ohs.phase2.visibility;

/**
 * Sun angle class calculates the altitude of the 
 * sun at a given time and date, for example anything over
 * -18 degrees is considered twilight an observable time.
 *
 * slaRdplan - Sun position
 * slaDe2h - Sun eleveation
 *
 * @version 25 July 2001
 * @author C. Mavros for ESO
 */

import org.apache.log4j.Logger;

public class SunAngle extends JNIWrapper{

   /**
    * Standard log channel
    * <br>ChangeLog: cmavros 21-12-01 added
    */
   private static Logger stdlog_ =
   Logger.getLogger( SunAngle.class );

    /**
    * The constant that the calling i
    * SLALIB function uses for the Sun
    */
    private static final int SUN = 0;

    /**
    * Position in RA and DEC position in the
    * array returned from the SLALIB function
    */
    private static final int SUN_RA = 0;
    private static final int SUN_DEC = 1;

    /**
    * 180/PI  Convert radians to degrees 
    */
    private static final double R2D = 57.2957795130823;

    /**
     * pi/12:  hours to radians
     */
    private static final double H2R  = 0.2617993877992;


    /**
     * getSunAngle calls slaRdplan for the Sun RA and Dec for 
     * calculating the hour angle(HA) then calling slaDe2h to 
     * transform the HA and Dec into Azimuth and Elevation. 
     * Elevation which is the sun twilight angle.
     *
     * Note: Included is an another calculation for sun angle
     * taken for the Skycalc program to use as a check
     * if neccessary.
     *
     * @param jDate Julian Date of OB 
     * @param longitude of observatory (radians) 
     * @param lat of observatory  (radians)
     * @param lst (radians)
     * @return the sun angular separation distance in degrees 
     */

    public static double getSunAngle(double jDate, double longitude, 
      double lat, double lst){

        double rdpSun[] = new double[3];
        double rdpDe2h[] = new double[2];

        /** Calculate the RA and Dec of the Sun */
        rdpSun  = jnislaRdplan(jDate - 2400000.5, SUN, longitude, lat);

        //stdlog_.info("visiplot.SunAngle.getSunAngle sunRA "+ rdpSun[SUN_RA]/H2R);
        //stdlog_.info("visiplot.SunAngle.getSunAngle sunDEC "+ rdpSun[SUN_DEC]*R2D);

        /** Hour Angle - The angle between an observer's meridian and the 
         * hour circle on which some celestial body lies.
         */
        double HA = lst - rdpSun[SUN_RA];

        /** Sun twilight angle in radians */
        rdpDe2h  = jnislaDe2h(HA, rdpSun[SUN_DEC], lat);
        double result = rdpDe2h[1]*R2D;
        //stdlog_.info("visiplot.SunAngle.getSunAngle = "+ result);

        /* Sanity check calculation Sun twilight angle 
           double result = Math.asin(Math.sin(lat) * Math.sin(rdpSun[SUN_DEC]) + 
             Math.cos(lat) * Math.cos(rdpSun[SUN_DEC]) * Math.cos(HA));
           stdlog_.info("visiplot.SunAngle.getSunAngle = "+ result*R2D);
        */

        return(result);   
    }
}
