package org.eso.ohs.phase2.visibility;

import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.phase2.visiplot.VisiPlotOptions;

/*
 * This class contains the constraints for visibility calculation
 * 
 * Created on Aug 19, 2005
 * 
 * @author pnunes 
 *
 */
public class VisicalcOptions {
	
	/**
     * Config keyword for telescope zenith constraint
     */
    public static final String telescopeConstraintKeyword = "TELESCOPE.ELEVATION.CONSTRAINT";
    
    /*
     * Config keyword for precision of zenith constraint calculation
     */
    public static final String telescopeConstraintPrecisionKeyword = "TELESCOPE.ELEVATION.CONSTRAINT.PRECISION";
	
	private double moonDistanceTolerance; // tolerance of moon distance to target in percentage
	private double airmassTolerance; // airmass tolerance in percentage
	private double fliTolerance; // fli tolerance in percentage
	private double sunConstraint;  // sun elevation angle in degrees
	private double moonConstraint;  // moon elevation angle in degrees
	private double zenithConstraint;  // maximum elevation in degrees
	private int zenithConstraintCalcPrecision; //precision with which the zenith constraint will be computed (in minutes)
	
	private boolean calcFli;
	private boolean calcMoonDist;
	private boolean calcMoon; 
	private boolean calcSun;
	private boolean calcTimeInterval; 
	private boolean calcSiderealTimeInterval;
	private boolean calcZenithInvisibility;
	
	/**
	 * Default constructor 
	 */
	public VisicalcOptions() {}
	
	/**
	 * Constructs a VisicalcOptions object from a VisiPlotOptions
	 * @param vpo
	 */
	public VisicalcOptions(VisiPlotOptions vpo) {
		        
		
		// since visiplot options doesn't define any tolerances, these are set to zero
		moonDistanceTolerance = 0; 
		airmassTolerance = 0;
		fliTolerance = 0;		
		
		
		// set zenith constraint to be considered by default
		calcZenithInvisibility = vpo.isEnabled(VisiPlotOptions.zenithConKey);
								
		// set zenith precision constraint default to 1 minute
		zenithConstraintCalcPrecision = new Integer(AppConfig.getAppConfig().getValue(telescopeConstraintPrecisionKeyword)).intValue(); 
		
		// read zenith constraint from config file
		zenithConstraint = new Double(AppConfig.getAppConfig().getValue(telescopeConstraintKeyword)).doubleValue();
		
		String val = vpo.getValue(VisiPlotOptions.sunConKey);
        sunConstraint = getSunConstraint(vpo, val);		
        val = vpo.getValue(VisiPlotOptions.moonDistConKey);
        moonConstraint = getMoonConstraint(vpo, val);
        
        
        calcFli = vpo.isEnabled(VisiPlotOptions.fliConEnabledKey);        	
        calcMoonDist = vpo.isEnabled(VisiPlotOptions.moonDistConEnabledKey);
        
        if (calcFli || calcMoonDist) {
        	calcMoon = true;
        } else {
        	calcMoon = false;
        }
                
        calcSun = vpo.isEnabled(VisiPlotOptions.sunConEnabledKey);
        calcTimeInterval =
            vpo.isEnabled(VisiPlotOptions.obtimeIntervalConKey);
        calcSiderealTimeInterval =
            vpo.isEnabled(VisiPlotOptions.obSiderealTimeIntervalConKey);

 
	}
	
	
	public double getAirmassTolerance() {
		return airmassTolerance;
	}
	public void setAirmassTolerance(double airmassTolerance) {
		this.airmassTolerance = airmassTolerance;
	}
	public boolean calcFli() {
		return calcFli;
	}
	public void setCalcFli(boolean calcFli) {
		this.calcFli = calcFli;
	}
	public boolean calcMoon() {
		return calcMoon;
	}
	public void setCalcMoon(boolean calcMoon) {
		this.calcMoon = calcMoon;
	}
	public boolean calcMoonDist() {
		return calcMoonDist;
	}
	public void setCalcMoonDist(boolean calcMoonDist) {
		this.calcMoonDist = calcMoonDist;
	}
	public boolean calcSiderealTimeInterval() {
		return calcSiderealTimeInterval;
	}
	public void setCalcSiderealTimeInterval(boolean calcSiderealTimeInterval) {
		this.calcSiderealTimeInterval = calcSiderealTimeInterval;
	}
	public boolean calcSun() {
		return calcSun;
	}
	public void setCalcSun(boolean calcSun) {
		this.calcSun = calcSun;
	}
	public boolean calcTimeInterval() {
		return calcTimeInterval;
	}
	public void setCalcTimeInterval(boolean calcTimeInterval) {
		this.calcTimeInterval = calcTimeInterval;
	}
	public double getFliTolerance() {
		return fliTolerance;
	}
	public void setFliTolerance(double fliTolerance) {
		this.fliTolerance = fliTolerance;
	}
	public double getMoonConstraint() {
		return moonConstraint;
	}
	public void setMoonConstraint(double moonConstraint) {
		this.moonConstraint = moonConstraint;
	}
	public double getMoonDistanceTolerance() {
		return moonDistanceTolerance;
	}
	public void setMoonDistanceTolerance(double moonDistanceTolerance) {
		this.moonDistanceTolerance = moonDistanceTolerance;
	}
	public double getSunConstraint() {
		return sunConstraint;
	}
	public void setSunConstraint(double sunConstraint) {
		this.sunConstraint = sunConstraint;
	}
	
	
	
	
	public boolean calcZenithInvisibility() {
		return calcZenithInvisibility;
	}

	public void setCalcZenithInvisibility(boolean calcZenithInvisibility) {
		this.calcZenithInvisibility = calcZenithInvisibility;
	}

	
	
	
	public double getZenithConstraint() {
		return zenithConstraint;
	}

	public void setZenithConstraint(double zenithConstraint) {
		this.zenithConstraint = zenithConstraint;
	}

	/**
	 * getSunConstraint calls isReadFromOB() to determines if the default value
	 * of -18 or the VisiPlot Options value is used
	 * 
	 * @return Sun Constraint
	 * @see #readVisiPlotOptions(ObservationBlock ob, VisiPlotOptions vpo)
	 * @see VisiPlotOptions#isReadFromOB(String value)
	 */
	private double getSunConstraint(VisiPlotOptions vpo, String value) {
		boolean readFromOB = VisiPlotOptions.isReadFromOB(value);
		double sunConstraint;

		if (!readFromOB) {
			try {
				sunConstraint = Double.valueOf(value).doubleValue();
			} catch (NumberFormatException e) {
				sunConstraint = -18.;
			}
		} else {
			sunConstraint = -18.;
		}
		// System.out.print(" visiplot.Visiplot.getSunConstraint " +
		// sunConstraint_);
		return (sunConstraint);
	}
    
    
    /**
	 * getMoonDistance calls isReadFromOB() to determines which constraint value
	 * to use; the VisiPlot Options value or the constraint from the database.
	 * 
	 * @return Moon Distance Constraint
	 * @see #readVisiPlotOptions(ObservationBlock ob, VisiPlotOptions vpo)
	 * @see VisiPlotOptions#isReadFromOB(String value)
	 */
	private double getMoonConstraint(VisiPlotOptions vpo,
			String value) {

		boolean readFromOB = VisiPlotOptions.isReadFromOB(value);
		double moonConstraint;

		if (!readFromOB) {
			try {
				moonConstraint = Double.valueOf(value).doubleValue();
			} catch (NumberFormatException e) {
				moonConstraint = 0.;
			}
		} else {
			moonConstraint = 0.;
		}
		//stdlog_.info(" visiplot.Visiplot.getMoonConstraint " + moonConstraint_ + " " + value);
		return (moonConstraint);
	}

	public int getZenithConstraintCalcPrecision() {
		return zenithConstraintCalcPrecision;
	}

	public void setZenithConstraintCalcPrecision(
			int zenithConstraintCalcPrecision) {
		this.zenithConstraintCalcPrecision = zenithConstraintCalcPrecision;
	}
    
    
    
	
}
