package org.eso.ohs.phase2.visiplot;


import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;

/**
 * Class TimeLog
 * 
 * This class is used to time the execution of an algorithm
 * It is useful for benchmarks
 * 
 * Example use:
 * 
 * String[] splitString = {"part1","part2"}
 * TimerLog timer = new TimerLog(splitString);
 * 
 * timer.calcTimer(1); // beginning of computation part1
 *  ... 
 *  ...             
 * timer.calcTimer(1);  // end of computation part1
 * timer.calcTimer(2); // beginning of computation part2
 *  ... 
 *  ...             
 * timer.calcTimer(2);  // end of computation part2
 * 
 * System.out.println(timer.reportResult());
 *  
 *  This will print a report like the following:
 *  
 *  Testing time (ms): 1089 Total calculated time:1000 Percentage of calculated time:99 
 * Name               value(ms)      percentage  numIterations  timePerIteration 
 * part1                651                  65.00     65        1
 * part2                349                  35.00     1         35 
 * 
 * @version $Revision: 1.5 $
 * @author ddorigo, Jul 18, 2005
 */

// $Id: TimerLog.java,v 1.5 2005/11/30 16:33:08 ddorigo Exp $
public class TimerLog {
    long[] startList_;

    long[] endList_;
    int[] callingIterations_;

    long[] partialSum_;

    boolean[] timingList_;

    private String[] intervalTitles_;

    private long starting_;

    private boolean calculate_ = false;
    /**
     * 
     *  
     */
    public TimerLog(String[] splitIntervals) {
        intervalTitles_ = splitIntervals;
        int numIntervals = splitIntervals.length;
        startList_ = new long[numIntervals];
        endList_ = new long[numIntervals];
        partialSum_ = new long[numIntervals];
        callingIterations_ = new int[numIntervals];
        timingList_ = new boolean[numIntervals];
        for (int i = 0; i < numIntervals; i++) {
            startList_[i] = 0;
            endList_[i] = 0;
            callingIterations_[i] = 0;
            partialSum_[i] = 0;
            timingList_[i] = false;
        }
        starting_ = System.currentTimeMillis();
        setCalculate(true);
    }

    public void startCalcTimer(int key) throws ObjectIOException {
        callingIterations_[key]++;
        if (isCalculate()){
        if (!timingList_[key]) {
            startList_[key] = System.currentTimeMillis();
            timingList_[key] = true;
        } else {
            throw new ObjectIOException("error in timing");
        }
        }
    }

    public void calcTimer(int key) {
        callingIterations_[key]++;
        if (isCalculate()){
        if (!timingList_[key]) {
            startList_[key] = System.currentTimeMillis();
            timingList_[key] = true;           
        } else {
            endList_[key] = System.currentTimeMillis();
            partialSum_[key] = endList_[key] - startList_[key]
                    + partialSum_[key];
            timingList_[key] = false;
        }
    }
    }
    public void endCalcTimer(int key) throws ObjectIOException {
        callingIterations_[key]++;
        if (isCalculate()){
        if (timingList_[key]) {
            endList_[key] = System.currentTimeMillis();
            partialSum_[key] = endList_[key] - startList_[key]
                    + partialSum_[key];
            timingList_[key] = false;
        } else {
            throw new ObjectIOException("error in timing");
        }
    }
    }

    public String reportResult() {
        long testingTime = System.currentTimeMillis() - starting_;
       
        StringBuffer result = new StringBuffer();
        if (testingTime > 0){
        long totalTime = 0;
        for (int i = 0; i < partialSum_.length; i++) {
            totalTime = partialSum_[i] + totalTime;
        }
      
        result.append("Testing time (ms): " + testingTime
                + " Total calculated time:"+totalTime+" Percentage of calculated time:" + (totalTime * 100)
                / testingTime + "\n\n");
        result
                .append("Name               value(ms)      percentage  numIterations  timePerIteration \n\n");

        for (int i = 0; i < intervalTitles_.length; i++) {
            result.append(TextUtils.padString(intervalTitles_[i], 25));
            result.append(TextUtils.padString(String.valueOf(partialSum_[i]),
                    20));
            double percentage = 0;
            double timePerIteration = partialSum_[i]/(double)callingIterations_[i];
            if (totalTime != 0) {
                percentage = (partialSum_[i] * 100) / (double)totalTime;
            }
            result.append(getTrimString(percentage)+"     ");
            result.append(TextUtils.padString(String
                    .valueOf(callingIterations_[i]/2), 10));
            String timeItString = getTrimString(timePerIteration);
            result.append(timeItString);
            result.append("\n");
        } 
        }else {
            result.append("Testing time (ms) is 0; cannot test ");
        }

        return result.toString();
    }

    /**
     * @param timePerIteration
     * @return String
     */
    private String getTrimString(double timePerIteration) {
        String timeItString = Double.toString(timePerIteration);
        if (timeItString.length() > 5) {
            timeItString = timeItString.substring(0,5);
        } else {
            timeItString = TextUtils.padString(timeItString,5);
        }
        return timeItString;
    }

    /**
     * @param calculate_ The calculate_ to set.
     */
    public void setCalculate(boolean calculate) {
        this.calculate_ = calculate;
    }

    /**
     * @return Returns the calculate_.
     */
    public boolean isCalculate() {
        return calculate_;
    }
}